﻿// ----------------------------------------------------------------------------------------
// Author:                    Joaquín López y Magaly Vargas 
// Company:                   ALBALIA INTERACTIVA S.L.
// Assembly version:          1.0.0.0
// Date:                      05/02/2009
// Time:                      17:02
// Solution Name:             FacturaeAdd-in
// Solution Filename:         FacturaeAdd-in.sln
// Solution FullFilename:     C:\Ribbon_Facturae\FacturaeAdd-in.sln
// Project Name:              FacturaeAdd-in
// Project Filename:          FacturaeAdd-in.csproj
// Project FullFilename:      C:\Ribbon_Facturae\FacturaeAdd-in\FacturaeAdd-in.csproj
// Project Item Name:         ValueBinder.cs
// Project Item Filename:     ValueBinder.cs
// Project Item FullFilename: C:\Ribbon_Facturae\FacturaeAdd-in\FacturaeFiles\ValueBinder.cs
// Project Item Kind:         Código
// Purpose:                   Vincula los datos de una facturae (archivo XML) a la plantilla de Word Facturae
//Copyright © 2008 - 2009 Albalia Interactiva S.L. Reservados todos los derechos.
// ----------------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Office.Interop.Word;
using System.Windows.Forms;
using System.Globalization;
using System.Reflection;
using System.Xml.Serialization;

namespace FacturaeAdd_in.FacturaeFiles
{
    /// <summary>
    /// Clase utilizada para la importación de archivos en formato xml facturae a la plantilla de Word Facturae
    /// Básicamente hace lo inverso a FacturaeGenerator
    /// </summary>
    class ValueBinder
    {

        /// <summary>
        /// Objeto de tipo Facturae resultante de deserealizar un archivo de faturae en XML, del que se tomará la información para vincularla al documento de word
        /// </summary>
        Facturae facturae;

        /// <summary>
        /// Contiene la información de cabecera del archivo de facturae
        /// </summary>
        FileHeaderType fileHeader;

        /// <summary>
        /// Listado de ContentControls del documento de Word al que se vincularan los datos
        /// </summary>
        ContentControls controls;

        /// <summary>
        /// Contiene la información del encabezado de factura
        /// </summary>
        InvoiceHeaderType invoiceHeader;

        /// <summary>
        /// Contiene la infomacion del ó emisor de la factura
        /// </summary>
        BusinessType sellerParty;

        /// <summary>
        /// Contiene la Información del cliente o receptor de la factura
        /// </summary>
        BusinessType buyerParty;

        /// <summary>
        /// Contiene la información referente al tercero si la factura fue emitida por un ente externo
        /// </summary>
        ThirdPartyType thirdParty;


        /// <summary>
        /// Contiene la información de la factura
        /// </summary>
        InvoiceType invoice;

        /// <summary>
        /// Contiene los datos de emisión de la factura
        /// </summary>
        InvoiceIssueDataType invoiceIssueData;

        /// <summary>
        /// Listado de detalles de la factura
        /// </summary>
        List<InvoiceLineType> items;

        /// <summary>
        /// Listado de descuentos de la factura
        /// </summary>
        List<DiscountType> discounts;

        /// <summary>
        /// Contiene la información de los detalles de la factura
        /// </summary>
        InvoiceTotalsType invoiceTotals;

        /// <summary>
        /// Referencia al documento de word sobre el que se importaran los datos
        /// </summary>
        Microsoft.Office.Interop.Word.Document document;

        /// <summary>
        /// array de caracteres a eliminar de los valores tomados directamente de las celdas de una table
        /// </summary>
        char[] x = { '\r', '\a', '\n' };

        /// <summary>
        /// Contienen los indices de las tablas de la plantilla de word facturae
        /// </summary>
        int t2, t3, t4, t5, t6, t7;

        /// <summary>
        /// formato en el que viene la fecha en los archivos ed facturae.xml
        /// </summary>
        string[] format = {"dd/MM/yyyy"};

        /// <summary>
        /// indica si se importan los datos para rectificar una factura
        /// </summary>
        bool corrective;

        /// <summary>
        /// Referencia cultural utilizada para interpretar los valores numéricos del archivo xml
        /// </summary>
        CultureInfo culture;   

        /// <summary>
        /// formato a utiliar para las conversiones numéricas
        /// </summary>
        NumberFormatInfo num;

        /// <summary>
        /// Referencia cultural instalada en el SO, utilizada para visualizar los valores en el documentos
        /// </summary>
        CultureInfo cultureInvoice;

        /// <summary>
        /// Formato a utilziar para el despliegue de números en el documento de Word facturae
        /// </summary>
        NumberFormatInfo numberFormat;


        /// <summary>
        /// Importa los datos de un objeto Facturae al documento de plantilla Word facturae
        /// </summary>
        /// <param name="facturae">Facturae que contiene los datos a visualizar</param>
        /// <param name="document">Documento de word sobre el que se visualizarán los datos</param>
        /// <param name="corrective">Especifica si los datos importados son para generar  una factura rectificativa</param>
        /// <param name="cultureInvoice">Referencia cultural a utilziar para visulizar los datos</param>

        public ValueBinder(Facturae facturae, Microsoft.Office.Interop.Word.Document document, bool corrective,CultureInfo cultureInvoice)
        {
            this.facturae = facturae;
            this.document = document;
            this.controls = this.document.ContentControls;
            this.corrective = corrective;

            //this.cultureInvoice = cultureInvoice;
            this.cultureInvoice = CultureInfo.InstalledUICulture;

            this.numberFormat = cultureInvoice.NumberFormat;

            culture = new CultureInfo("en-US");
            num = culture.NumberFormat;
            num.NumberDecimalSeparator = ".";
                
            enumerateTables();
            

            if(facturae!=null)
            {
                //inicializar elementos 
                fileHeader = facturae.FileHeader;
                invoiceHeader = facturae.Invoices[0].InvoiceHeader;
                sellerParty = facturae.Parties.SellerParty;
                buyerParty=facturae.Parties.BuyerParty;
                thirdParty = facturae.FileHeader.ThirdParty;
                //paymentsDetails = facturae.Invoices[0].PaymentDetails[0];
                invoice = facturae.Invoices[0];
                invoiceIssueData = invoice.InvoiceIssueData;
                items = invoice.Items;
                discounts = invoice.InvoiceTotals.GeneralDiscounts;
                invoiceTotals = invoice.InvoiceTotals;
                //if (facturae.Invoices[0].InvoiceHeader.Corrective != null)
                //    this.corrective = true;
                //cargar datos

                SetInvoiceHeaderData();
                SetIssuerPartyData();
                setBuyerPartyData();
                
                try
                {
                    setThirdPartyData();
                }
                catch (Exception)
                {
                    ControlMessages msg = ControlMessages.getInstance();
                    throw new Exception(msg.getMessage(ControlMessages.Msg.InfoThirdParty));
                }
                setAdditionalInformationData();
                setInvoiceIssueData();
                setItemsData();
                setDiscountData();
                setTotalsData();
                //CleanTemplate(facturae);

            }
        }

        /// <summary>
        /// Vincula al documento los datos correspondientes al encabezado de la factura
        /// </summary>
        private void SetInvoiceHeaderData()
        {
            if (invoiceHeader != null)
            {
                if (!corrective)
                {
                    try
                    {
                        setControlValue(ControlList.Tag.IH_InvoiceSeriesCode, invoiceHeader.InvoiceSeriesCode);
                    }
                    catch (Exception)
                    {
                    }
                    setControlValue(ControlList.Tag.IH_InvoiceNumber, invoiceHeader.InvoiceNumber);

                    if (invoiceHeader.Corrective != null)
                    {
                        try
                        {
                            setControlValue(ControlList.Tag.CORR_InvoiceSeriesCode, invoiceHeader.Corrective.InvoiceSeriesCode);
                        }
                        catch (Exception)
                        {
                        }
                        setControlValue(ControlList.Tag.CORR_InvoiceNumber, invoiceHeader.Corrective.InvoiceNumber);
                    }

                    if (invoiceHeader.Corrective != null)
                    {
                        setCorrective(invoiceHeader.Corrective);                        
                    }
                  
                }
                else
                {

                    try
                    {
                        setControlValue(ControlList.Tag.CORR_InvoiceSeriesCode, invoiceHeader.InvoiceSeriesCode);
                    }
                    catch (Exception)
                    {
                    }
                        setControlValue(ControlList.Tag.CORR_InvoiceNumber, invoiceHeader.InvoiceNumber);
                    

                }
            }
        }

        /// <summary>
        ///  Vincula al documento los datos correspondientes al emisor de la factura
        /// </summary>
        private void SetIssuerPartyData()
        {
            if (sellerParty != null)
            {
                getControl(ControlList.Tag.SP_PersonTypeCode).Range.Text = controlValue(ControlList.Tag.SP_PersonTypeCode, sellerParty.TaxIdentification.PersonTypeCode.ToString("g"));
               getControl(ControlList.Tag.SP_ResidenceTypeCode).Range.Text = controlValue(ControlList.Tag.SP_ResidenceTypeCode,sellerParty.TaxIdentification.ResidenceTypeCode.ToString("g"));
                setControlValue(ControlList.Tag.SP_TaxIdentificationNumber, sellerParty.TaxIdentification.TaxIdentificationNumber);

                if (sellerParty.TaxIdentification.PersonTypeCode.Equals(PersonTypeCodeType.F))
                {
                    IndividualType individual = (IndividualType)sellerParty.Item;
                    setControlValue(ControlList.Tag.SP_Name, individual.Name);
                    setControlValue(ControlList.Tag.SP_FirstSurname, individual.FirstSurname);
                    setControlValue(ControlList.Tag.SP_SecondSurname, individual.SecondSurname);

                    setSP_AddressData(individual.Item);

                    try
                    {
                        setControlValue(ControlList.Tag.SP_Telephone, individual.ContactDetails.Telephone);
                    }
                    catch (Exception)
                    {
                        
                    }
                    try
                    {
                        setControlValue(ControlList.Tag.SP_email, individual.ContactDetails.ElectronicMail);
                    }
                    catch (Exception)
                    {
                        
                    }

                }
                else
                {
                    LegalEntityType legalentity = (LegalEntityType)sellerParty.Item;
                    setControlValue(ControlList.Tag.SP_CorporateName, legalentity.CorporateName);

                    setSP_AddressData(legalentity.Item);

                    try
                    {
                        setControlValue(ControlList.Tag.SP_Telephone, legalentity.ContactDetails.Telephone);
                    }
                    catch (Exception)
                    {
                    }
                    try
                    {
                        setControlValue(ControlList.Tag.SP_email, legalentity.ContactDetails.ElectronicMail);
                    }
                    catch (Exception)
                    {
                    }
                    
                }
            }
        }

        /// <summary>
        ///  Vincula al documento los datos correspondientes al cliente o receptor de la factura
        /// </summary>
        private void setBuyerPartyData()
        {
            if (buyerParty != null)
            {
                getControl(ControlList.Tag.BP_PersonTypeCode).Range.Text = controlValue(ControlList.Tag.BP_PersonTypeCode, buyerParty.TaxIdentification.PersonTypeCode.ToString("g"));
                getControl(ControlList.Tag.BP_ResidenceTypeCode).Range.Text = controlValue(ControlList.Tag.BP_ResidenceTypeCode, buyerParty.TaxIdentification.ResidenceTypeCode.ToString("g"));
                setControlValue(ControlList.Tag.BP_TaxIdentificationNumber, buyerParty.TaxIdentification.TaxIdentificationNumber);

                if (buyerParty.TaxIdentification.PersonTypeCode.Equals(PersonTypeCodeType.F))
                {
                    IndividualType individual = (IndividualType)buyerParty.Item;
                    setControlValue(ControlList.Tag.BP_Name, individual.Name);
                    setControlValue(ControlList.Tag.BP_FirstSurname, individual.FirstSurname);
                    setControlValue(ControlList.Tag.BP_SecondSurname, individual.SecondSurname);

                    setBP_AddressData(individual.Item);

                    try
                    {
                        setControlValue(ControlList.Tag.BP_Telephone, individual.ContactDetails.Telephone);
                    }
                    catch (Exception)
                    {
                    }
                    try
                    {
                        setControlValue(ControlList.Tag.BP_email, individual.ContactDetails.ElectronicMail);
                    }
                    catch (Exception)
                    {
                    }

                }
                else
                {
                    LegalEntityType legalentity = (LegalEntityType)buyerParty.Item;
                    setControlValue(ControlList.Tag.BP_CorporateName, legalentity.CorporateName);

                    setBP_AddressData(legalentity.Item);

                    try
                    {
                        setControlValue(ControlList.Tag.BP_Telephone, legalentity.ContactDetails.Telephone);
                    }
                    catch (Exception)
                    {
                    }
                    try
                    {
                        setControlValue(ControlList.Tag.BP_email, legalentity.ContactDetails.ElectronicMail);
                    }
                    catch (Exception)
                    {
                    }

                }
            }
        }

        /// <summary>
        ///  Vincula al documento los datos correspondientes al tercero, en caso de que el emisor de la factura, sea un ente ajeno a las partes principales
        /// </summary>
        private void setThirdPartyData()
        {
            if (thirdParty != null)
            {
                getControl(ControlList.Tag.TH_PersonTypeCode).Range.Text = controlValue(ControlList.Tag.TH_PersonTypeCode, thirdParty.TaxIdentification.PersonTypeCode.ToString("g"));
                getControl(ControlList.Tag.TH_ResidenceTypeCode).Range.Text = controlValue(ControlList.Tag.TH_ResidenceTypeCode, thirdParty.TaxIdentification.ResidenceTypeCode.ToString("g"));
                setControlValue(ControlList.Tag.TH_TaxIdentificationNumber, thirdParty.TaxIdentification.TaxIdentificationNumber);

                if (thirdParty.TaxIdentification.PersonTypeCode.Equals(PersonTypeCodeType.F))
                {
                    IndividualType individual = (IndividualType)thirdParty.Item;
                    setControlValue(ControlList.Tag.TH_Name, individual.Name);
                    setControlValue(ControlList.Tag.TH_FirstSurname, individual.FirstSurname);
                    setControlValue(ControlList.Tag.TH_SecondSurname, individual.SecondSurname);

                    setTH_AddressData(individual.Item);

                    try
                    {
                        setControlValue(ControlList.Tag.TH_Telephone, individual.ContactDetails.Telephone);
                    }
                    catch (Exception)
                    {
                    }
                    try
                    {
                        setControlValue(ControlList.Tag.TH_Email, individual.ContactDetails.ElectronicMail);
                    }
                    catch (Exception)
                    {
                    }

                }
                else
                {
                    LegalEntityType legalentity = (LegalEntityType)thirdParty.Item;
                    setControlValue(ControlList.Tag.TH_CorporateName, legalentity.CorporateName);

                    setTH_AddressData(legalentity.Item);

                    try
                    {
                        setControlValue(ControlList.Tag.TH_Telephone, legalentity.ContactDetails.Telephone);
                    }
                    catch (Exception)
                    {
                    }
                    try
                    {
                        setControlValue(ControlList.Tag.TH_Email, legalentity.ContactDetails.ElectronicMail);
                    }
                    catch (Exception)
                    {
                    }

                }
            }
        }

        /// <summary>
        ///  Vincula al documento los datos correspondientes a la dirección del emisor de la factura
        /// </summary>
        /// <param name="addressItem">objeto que contiene la información de la dirección</param>
        private void setSP_AddressData(object addressItem)
        {
            try
            {
                AddressType address = (AddressType)addressItem;
                setControlValue(ControlList.Tag.SP_Address, address.Address);
                setControlValue(ControlList.Tag.SP_Province, address.Province);
                setControlValue(ControlList.Tag.SP_PostCode, address.PostCode);
                getControl(ControlList.Tag.SP_CountryCode).Range.Text = controlValue(ControlList.Tag.SP_CountryCode,address.CountryCode.ToString("g"));
                setControlValue(ControlList.Tag.SP_Town, address.Town);

            }
            catch (Exception)
            {
                OverseasAddressType OvAddress = (OverseasAddressType)addressItem;
                string postCode = ""; string Town = "";
                try
                {
                    foreach (char c in OvAddress.PostCodeAndTown.ToCharArray())
                    {
                        int n = int.Parse(c.ToString());
                        postCode += c.ToString();
                    }
                }
                catch (FormatException)
                {
                    Town = OvAddress.PostCodeAndTown.Substring(postCode.Length);
                }
                setControlValue(ControlList.Tag.SP_Address, OvAddress.Address);
                setControlValue(ControlList.Tag.SP_Province, OvAddress.Province);
                setControlValue(ControlList.Tag.SP_PostCode, postCode);
                setControlValue(ControlList.Tag.SP_Town, Town);
                getControl(ControlList.Tag.SP_CountryCode).Range.Text = controlValue(ControlList.Tag.SP_CountryCode, OvAddress.CountryCode.ToString("g"));


            }
        }

        /// <summary>
        ///  Vincula al documento los datos correspondientes a la dirección del receptor de la factura
        /// </summary>
        /// <param name="addressItem">objeto que contiene la información de la dirección</param>
        private void setBP_AddressData(object addressItem)
        {
            try
            {
                AddressType address = (AddressType)addressItem;
                setControlValue(ControlList.Tag.BP_Address, address.Address);
                setControlValue(ControlList.Tag.BP_Province, address.Province);
                setControlValue(ControlList.Tag.BP_PostCode, address.PostCode);
                getControl(ControlList.Tag.BP_CountryCode).Range.Text = controlValue(ControlList.Tag.BP_CountryCode, address.CountryCode.ToString("g"));

                setControlValue(ControlList.Tag.BP_Town, address.Town);

            }
            catch (Exception)
            {
                {
                    OverseasAddressType OvAddress = (OverseasAddressType)addressItem;
                    string postCode = ""; string Town = "";
                    try
                    {
                        foreach (char c in OvAddress.PostCodeAndTown.ToCharArray())
                        {
                            int n = int.Parse(c.ToString());
                            postCode += c.ToString();
                        }
                    }
                    catch (FormatException)
                    {
                        Town = OvAddress.PostCodeAndTown.Substring(postCode.Length);
                    }

                    setControlValue(ControlList.Tag.BP_Address, OvAddress.Address);
                    setControlValue(ControlList.Tag.BP_Province, OvAddress.Province);
                    setControlValue(ControlList.Tag.BP_PostCode, postCode);
                    setControlValue(ControlList.Tag.BP_Town, Town);
                    getControl(ControlList.Tag.BP_CountryCode).Range.Text = controlValue(ControlList.Tag.BP_CountryCode, OvAddress.CountryCode.ToString("g"));

                }
            }
        }

        /// <summary>
        ///  Vincula al documento los datos correspondientes a la dirección del tercero, cuando el emisor de la factura es un ente externo
        /// </summary>
        /// <param name="addressItem">objeto que contiene la información de la dirección</param>
        private void setTH_AddressData(object addressItem)
        {
            try
            {

                AddressType address = (AddressType)addressItem;
                setControlValue(ControlList.Tag.TH_PostCode, address.PostCode);
                setControlValue(ControlList.Tag.TH_Town, address.Town);
                setControlValue(ControlList.Tag.TH_Address, address.Address);
                setControlValue(ControlList.Tag.TH_Province, address.Province);
                getControl(ControlList.Tag.TH_CountryCode).Range.Text = controlValue(ControlList.Tag.TH_CountryCode, address.CountryCode.ToString("g"));



            }
            catch (Exception)
            {
                OverseasAddressType OvAddress = (OverseasAddressType)addressItem;
                string postCode = ""; string Town = "";
                try
                {
                    foreach (char c in OvAddress.PostCodeAndTown.ToCharArray())
                    {
                        int n = int.Parse(c.ToString());
                        postCode += c.ToString();
                    }
                }
                catch (FormatException)
                {
                    Town = OvAddress.PostCodeAndTown.Substring(postCode.Length);
                }

                setControlValue(ControlList.Tag.TH_Address, OvAddress.Address);
                setControlValue(ControlList.Tag.TH_Province, OvAddress.Province);
                setControlValue(ControlList.Tag.TH_PostCode, postCode);
                setControlValue(ControlList.Tag.TH_Town, Town);
                getControl(ControlList.Tag.TH_CountryCode).Range.Text = controlValue(ControlList.Tag.TH_CountryCode, OvAddress.CountryCode.ToString("g"));

            }
        }
              

        /// <summary>
        ///  Vincula al documento los datos correspondientes a la informacion adicional de la factura
        /// </summary>
        private void setAdditionalInformationData()
        {
            try
            {
                enumerateTables();
                document.Tables[t4].Rows[2].Range.Text = invoice.AdditionalData.InvoiceAdditionalInformation;
            }
            catch (Exception)
            {
            }
            
        }

        /// <summary>
        ///  Vincula al documento los datos correspondientes a los datos de emisión de la factura
        /// </summary>
        private void setInvoiceIssueData()
        {
            
            CultureInfo cult = new CultureInfo("es-ES");
            setControlValue(ControlList.Tag.InvIssue_IssueDate,invoiceIssueData.IssueDate.ToString(cult));            
            setControlValue(ControlList.Tag.InvIssue_OperationDate,invoiceIssueData.OperationDate.ToString(cult));
            setControlValue(ControlList.Tag.InvIssue_InvoiceCurrencyCode, facturae.FileHeader.Batch.InvoiceCurrencyCode.ToString("g"));
            setControlValue(ControlList.Tag.InvIssue_LanguageName, invoiceIssueData.LanguageName.ToString("g"));
            getControl(ControlList.Tag.InvoiceIssuerType).Range.Text=controlValue(ControlList.Tag.InvoiceIssuerType,fileHeader.InvoiceIssuerType.ToString("g"));
            if (!corrective)
                getControl(ControlList.Tag.IH_InvoiceClass).Range.Text = controlValue(ControlList.Tag.IH_InvoiceClass, invoiceHeader.InvoiceClass.ToString("g"));
            else
            {
                getControl(ControlList.Tag.IH_InvoiceClass).Range.Text = controlValue(ControlList.Tag.IH_InvoiceClass, InvoiceClassType.OR.ToString("g"));
            }
        }
        
        /// <summary>
        /// Busca el ContentControl llamado tag y le asigna value a la propiedad texto
        /// </summary>
        /// <param name="tag">Nombre del ContentControl</param>
        /// <param name="value">Valor a asignarle como texto al ContentControl tag</param>
        private void setControlValue(ControlList.Tag tag, string value)
        {

            try
            {
                foreach (Microsoft.Office.Interop.Word.ContentControl item in controls)
                {
                    if (!string.IsNullOrEmpty(item.Tag))
                    {
                        if (item.Tag.ToLower().Equals(tag.ToString("g").ToLower()))
                        {
                            item.Range.Text = value; break;
                        }

                    }
                }
            }
            catch (Exception) { }
        }

        /// <summary>
        /// Busca el ContentControl con el que tiene el tag especificado
        /// </summary>
        /// <param name="tag">tag del ContentControl buscado</param>
        /// <returns>el ContentControl con tag = tag, null si no lo encuentra</returns>
        private ContentControl getControl(ControlList.Tag tag)
        {
            Microsoft.Office.Interop.Word.ContentControl control = null;
            try
            {
                foreach (Microsoft.Office.Interop.Word.ContentControl item in controls)
                {
                    if (!string.IsNullOrEmpty(item.Tag))
                    {
                        if (item.Tag.ToLower().Equals(tag.ToString("g").ToLower()))
                        {
                            item.LockContents = false;
                           control = item;
                           break;
                        }

                    }
                }
            }
            catch (Exception) { return null; }

            return control;
        }


        /// <summary>
        /// método que se encarga de identificar los indices de las tablas.
        /// <para>
        /// Cada una de las tablas tiene un número en el encabezado, que permite identificarlas sin importar el orden en que aparezcan,
        /// se llama al inicio de cada operacion de cálculos,para aseguraserse de que las las variables de los indices esten actualizadas
        /// en caso de que una tabla haya sido eliminada
        /// 
        /// <example>
        /// se toma el texto del encabezado de la tabla, 
        /// 
        /// <code>string table = this.Tables[i].Rows[1].Cells[1].Range.Text.Trim(x).Trim();</code>
        /// 
        /// con base en el valor de "table", se asigna el indice a cada una de las variables para indices, t1,t2,t3,t4.t5,t6
        /// </example>
        /// </para>
        /// 
        /// </summary>       
        private void enumerateTables()
        {
            try
            {

                document = Globals.ThisAddIn.Application.ActiveDocument;

                int count = document.Tables.Count;
                if (count <= 0)
                {
                    ControlMessages msg = ControlMessages.getInstance();
                    throw new Exception(msg.getMessage(ControlMessages.Msg.ErrorNotValidTemplate));
                }

                else
                {

                    for (int i = 2; i <= document.Tables.Count; i++)
                    {
                        try
                        {
                            string table = document.Tables[i].Rows[1].Cells[1].Range.Text.Trim(x).Trim();
                            switch (table)
                            {

                                case "2":
                                    t2 = i; break;
                                case "3":
                                    t3 = i; break;
                                case "6":
                                    t4 = i; break;
                                case "4":
                                    t5 = i; break;
                                case "1":
                                    t6 = i; break;
                                case "7":
                                    t7 = i; break;

                            }
                        }
                        catch (Exception) { }
                    }
                }
            }
            catch (Exception) { }

        }

        /// <summary>
        /// Crea en la tabla detalles tantas filas como detalles tenga la facturae y asigna los respectivos valores
        /// </summary>
        private void setItemsData()
        {
            try
            {
                if (items.Count > 0)
                {
                    foreach (InvoiceLineType item in items)
                    {
                        object rowBefore = document.Tables[t2].Rows.Last;
                        document.Tables[t2].Rows.Add(ref rowBefore);

                        document.Tables[t2].Rows.Last.Previous.Cells[1].Range.Text = item.TransactionDate.ToString(format[0], culture);
                        document.Tables[t2].Rows.Last.Previous.Cells[2].Range.Text = item.ItemDescription;

                        document.Tables[t2].Rows.Last.Previous.Cells[3].Range.Text = String.Format(numberFormat, "{0:#,0.00}", double.Parse(item.Quantity, num));
                        document.Tables[t2].Rows.Last.Previous.Cells[4].Range.Text = String.Format(numberFormat, "{0:#,0.00}", double.Parse(item.UnitPriceWithoutTax, num));

                        double taxRate = 0;
                        double discountRate = 0;
                        double taxablebase = 0;

                        try
                        {
                            discountRate = double.Parse(item.DiscountsAndRebates[0].DiscountRate.Substring(0, item.DiscountsAndRebates[0].DiscountRate.LastIndexOf('.')), num);

                        }
                        catch (Exception)
                        {
                        }

                        document.Tables[t2].Rows.Last.Previous.Cells[5].Range.Text = string.Format(numberFormat, "{0:#,0.00}", discountRate);

                        taxRate = double.Parse(item.TaxesOutputs[0].TaxRate, num);

                        document.Tables[t2].Rows.Last.Previous.Cells[6].Range.Text = string.Format(numberFormat, "{0:#,0.00}", taxRate);


                        taxablebase = double.Parse(item.TaxesOutputs[0].TaxableBase.TotalAmount, num);
                        double irpfTaxRate = 0;


                        try
                        {
                            irpfTaxRate = double.Parse(item.TaxesWithheld[0].TaxRate, num);

                        }
                        catch (Exception) { }

                        try
                        {
                            taxablebase = double.Parse(item.GrossAmount, num);

                        }
                        catch (Exception) { }


                        document.Tables[t2].Rows.Last.Previous.Cells[7].Range.Text = string.Format(numberFormat, "{0:#,0.00}", irpfTaxRate);
                        document.Tables[t2].Rows.Last.Previous.Cells[8].Range.Text = string.Format(numberFormat, "{0:#,0.00}", (double.Parse(item.GrossAmount, num)));
                        document.Tables[t2].Rows.Last.Previous.Cells[9].Range.Text = string.Format(numberFormat, "{0:#,0.00}", (double.Parse(item.GrossAmount, num) + (double.Parse(item.GrossAmount, num) * taxRate / 100) - (double.Parse(item.GrossAmount, num) * irpfTaxRate / 100)));


                        try
                        {
                            setTaxType(taxRate, ((taxablebase * taxRate) / 100), taxablebase);
                        }
                        catch (Exception) { }
                    }
                }
            }
            catch (Exception) { }
        }

        /// <summary>
        /// Crea en la tabla de totales, una linea para la base imponible, y otra para la cuota por cada tipo difernete
        /// de impuestos repercutivos que haya en la facturae
        /// </summary>
        /// <param name="taxRate">Porcentaje del impuesto aplicado</param>
        /// <param name="taxamount">Importe del impuesto aplicado</param>
        /// <param name="taxableBase">Base sobre la que se aplica el impuesto</param>
        private void setTaxType(double taxRate, double taxamount, double taxableBase)
        {
            try
            {
                string TB = string.Concat("Base imponible (IVA ", taxRate.ToString(), ")");
                string TA = string.Concat("Cuota (IVA ", taxRate.ToString(), ")");


                int taxRows = document.Tables[t5].Rows.Last.Previous.Index - 3;
                bool asigned = false;

                if (taxRows >= 1)
                {
                    for (int i = 1; i < taxRows; i++)
                    {
                        string amount = document.Tables[t5].Rows[i + 3].Cells[2].Range.Text.Trim(x).Trim();
                        if (amount.ToLower().Equals(TB.ToLower()))
                        {
                            double tb = double.Parse(document.Tables[t5].Rows[i + 3].Cells[3].Range.Text.Trim(x).Trim(), numberFormat);
                            tb += taxableBase;

                            double ta = double.Parse(document.Tables[t5].Rows[i + 4].Cells[3].Range.Text.Trim(x).Trim(), numberFormat);
                            ta += taxamount;


                            if (ta != 0)
                            {

                                document.Tables[t5].Rows[i + 3].Cells[3].Range.Text = string.Format(numberFormat, "{0:#,0.00}", tb);
                                document.Tables[t5].Rows[i + 4].Cells[3].Range.Text = string.Format(numberFormat, "{0:#,0.00}", ta);
                            }

                            i++;
                            asigned = true;
                            break;
                        }

                    }
                    if (!asigned)
                    {
                        newTaxType(TB, TA, taxamount, taxableBase);
                    }

                }
            }
            catch (Exception) { }

        }

        /// <summary>
        /// Se utiliza para crear las lineas de los nuevos tipos de impuestos,  ya que los que tienen un mismo porcentaje, son acumulativos
        /// </summary>
        /// <param name="taxType">cadena utilzada para identificar la couta del tipo de impuesto</param>
        /// <param name="amountPrefix">cadena utiliada para identificar la base imponible del tipo de impuesto</param>
        /// <param name="taxAmount">importe del impuesto aplicado</param>
        /// <param name="taxableBase">base imponible sobre la que se aplica el impuesto</param>
        private void newTaxType(string taxType, string amountPrefix, double taxAmount, double taxableBase)
        {
            try
            {
                if (taxAmount != 0)
                {
                    object rowBefore = document.Tables[t5].Rows.Last.Previous;
                    document.Tables[t5].Rows.Add(ref rowBefore);
                    int index = document.Tables[t5].Rows.Last.Previous.Previous.Index;
                    document.Tables[t5].Rows[index].Cells[2].Range.Text = taxType;
                    document.Tables[t5].Rows[index].Cells[3].Range.Text = string.Format(numberFormat, "{0:#,0.00}", taxableBase);

                    rowBefore = document.Tables[t5].Rows.Last.Previous;
                    document.Tables[t5].Rows.Add(ref rowBefore);
                    index = document.Tables[t5].Rows.Last.Previous.Previous.Index;
                    document.Tables[t5].Rows[index].Cells[2].Range.Text = amountPrefix;
                    document.Tables[t5].Rows[index].Cells[3].Range.Text = string.Format(numberFormat, "{0:#,0.00}", taxAmount);
                }
            }
            catch (Exception) { }
        }


       /// <summary>
       /// Encargado de generar las nuevas lineas en la tabla de descuentos generales y establecer los respectivos valores
       /// </summary>
        private void setDiscountData()
        {
            try
            {
                //CultureInfo cult = new CultureInfo("es-ES");

                if (discounts.Count > 0)
                {
                    foreach (DiscountType item in discounts)
                    {
                        object rowBefore = document.Tables[t3].Rows.Last;
                        document.Tables[t3].Rows.Add(ref rowBefore);

                        document.Tables[t3].Rows.Last.Previous.Cells[1].Range.Text = item.DiscountReason;
                        document.Tables[t3].Rows.Last.Previous.Cells[2].Range.Text = item.DiscountRate.Substring(0, item.DiscountRate.LastIndexOf('.'));
                        document.Tables[t3].Rows.Last.Previous.Cells[3].Range.Text = string.Format(numberFormat, "{0:#,0.00}", double.Parse(item.DiscountAmount, num));

                    }
                }
            }
            catch (Exception) { }
        }

        /// <summary>
        /// Vincula la información de los totales de la factura, a la tabla de totales
        /// </summary>
        private void setTotalsData()
        {
            try{
                if (invoiceTotals != null)
                {
                    document.Tables[t5].Rows[2].Cells[3].Range.Text = String.Format(numberFormat, "{0:#,0.00}", double.Parse(invoiceTotals.TotalGrossAmount, num));
                    try
                    {
                        document.Tables[t5].Rows[3].Cells[3].Range.Text = String.Format(numberFormat, "{0:#,0.00}", double.Parse(invoiceTotals.TotalGeneralDiscounts, num));

                    }
                    catch (Exception)
                    {
                    }
                    document.Tables[t5].Rows.Last.Previous.Cells[3].Range.Text = String.Format(numberFormat, "{0:#,0.00}", double.Parse(invoiceTotals.TotalTaxesWithheld, num));
                    document.Tables[t5].Rows.Last.Cells[3].Range.Text = String.Format(numberFormat, "{0:#,0.00}", double.Parse(invoiceTotals.InvoiceTotal, num));

                }

            } catch(Exception){}

        }


        private string controlValue(ControlList.Tag tag, string selectedItem)
        {
            string value = "";

            if (string.IsNullOrEmpty(tag.ToString("g")))
                return value;

            /*buscamos el combobox indicado*/
            foreach (ContentControl item in document.ContentControls)
            {
                if (!string.IsNullOrEmpty(item.Tag))
                {
                    if (item.Tag.Equals(tag.ToString("g")))
                    {
                        /*si lo encontramos buscamos el valor correspondiente al texto seleccionado*/
                        foreach (ContentControlListEntry entry in item.DropdownListEntries)
                        {
                            if (!string.IsNullOrEmpty(item.Tag))
                            {
                                if (entry.Value.ToLower().Equals(selectedItem.ToLower()))
                                {
                                    value = entry.Text; break;
                                }
                            }

                        } break;
                    }
                }
            }
            return value;
        }

        private static string ConvertToString(Enum e)
        {
            // Get the Type of the enum
            Type t = e.GetType();

            // Get the FieldInfo for the member field with the enums name
            FieldInfo info = t.GetField(e.ToString("G"));

            // Check to see if the XmlEnumAttribute is defined on this field
            if (!info.IsDefined(typeof(XmlEnumAttribute), false))
            {
                // If no XmlEnumAttribute then return the string version of the enum.
                return e.ToString("G");
            }

            // Get the XmlEnumAttribute
            object[] o = info.GetCustomAttributes(typeof(XmlEnumAttribute), false);
            XmlEnumAttribute att = (XmlEnumAttribute)o[0];
            return att.Name;
        }
        
        public void CleanTemplate(Facturae facturae)
        {
            try
            {
                if (!facturae.FileHeader.InvoiceIssuerType.Equals(InvoiceIssuerTypeType.TE))
                {
                    document.Tables[t6].Delete();
                }
            }
            catch (Exception) { }
            


        }

        /// <summary>
        /// Si la clase de la factura es OR ó CR vincula al documento los datos de la rectificación
        /// </summary>
        /// <param name="correctiveData">Objeto que cntiene los datos de la rectificación</param>
        private void setCorrective(CorrectiveType correctiveData)
        {
            CorrectiveType correctiveD = correctiveData;

            try
            {
                getControl(ControlList.Tag.CORR_InvoiceNumber).Range.Text = correctiveD.InvoiceNumber;
                getControl(ControlList.Tag.CORR_InvoiceSeriesCode).Range.Text = correctiveD.InvoiceSeriesCode;
                getControl(ControlList.Tag.CORR_ReasonDescription).Range.Text = correctiveD.ReasonDescription;
                getControl(ControlList.Tag.CORR_TaxPeriod_StartDate).Range.Text = correctiveD.TaxPeriod.StartDate.ToString();
                getControl(ControlList.Tag.CORR_TaxPeriod_EndDate).Range.Text = correctiveD.TaxPeriod.EndDate.ToString();
                getControl(ControlList.Tag.CORR_CorrectionMethodDescription).Range.Text = correctiveD.CorrectionMethodDescription;
            }
            catch (Exception)
            {
                ControlMessages msg = ControlMessages.getInstance();

                throw new Exception(msg.getMessage(ControlMessages.Msg.InfoThirdParty));
            }

            
        }




    }
}
