﻿// ----------------------------------------------------------------------------------------
// Author:                    Joaquín López y Magally Vargas
// Company:                   Albalia Interactiva S.L
// Assembly version:          1.0.0.0
// Date:                      10/02/2009
// Time:                      12:03
// Solution Name:             FacturaeAdd-in
// Solution Filename:         FacturaeAdd-in.sln
// Solution FullFilename:     C:\Ribbon_Facturae\FacturaeAdd-in.sln
// Project Name:              FacturaeAdd-in
// Project Filename:          FacturaeAdd-in.csproj
// Project FullFilename:      C:\Ribbon_Facturae\FacturaeAdd-in\FacturaeAdd-in.csproj
// Project Item Name:         FrmInvoiceType.cs
// Project Item Filename:     FrmInvoiceType.cs
// Project Item FullFilename: C:\Ribbon_Facturae\FacturaeAdd-in\FrmInvoiceType.cs
// Project Item Kind:         Formulario
// Purpose:                   Permitir al usuario seleccionar el tipo de factura que desea emitir.(Electrónica o Papel)
//Copyright © 2008 - 2009 Albalia Interactiva S.L. Reservados todos los derechos.
// ----------------------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace FacturaeAdd_in
{
    /// <summary>
    /// Formulario que permite al usuario seleccionar el tipo de factura que se realizará,
    /// factura electrónica Facturae o factura en papel.
    /// </summary>
    public partial class FrmInvoiceType : Form
    {

        #region "Properties"

        /// <summary>
        /// Propiedad de tipo bool que indica si el usuario a seleccionado Facturae.
        /// </summary>
        public bool Facturae { get; set; }

        /// <summary>
        /// Propiedad de tipo bool que indica si el usuario a seleccionado factura en papel.
        /// </summary>
        public bool Paper { get; set; }

        #endregion

        #region "Constructors"

        /// <summary>
        /// Constructor que inicializa todos los componentes gráficos del formulario.
        /// </summary>
        public FrmInvoiceType()
        {
            InitializeComponent();
        }

        #endregion

        #region "EventHandler"


        /// <summary>
        /// Método que maneja el evento que se produce al hacer click con el ratón sobre el botón de aceptar.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnOk_Click(object sender, EventArgs e)
        {
            Facturae = rbtFacturae.Checked;
            Paper = rbtPaper.Checked;
            this.Close();
        }

        /// <summary>
        /// Método que maneja el evento que se produce al hacer click con el ratón sobre el botón de cancelar.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnCancel_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        /// <summary>
        /// Método que maneja el evento que se produce al mostrarse el formulario.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void FrmInvoiceType_Load(object sender, EventArgs e)
        {
            LoadLanguage();
            rbtFacturae.Checked = Facturae;
            rbtPaper.Checked = Paper;
            if (Facturae == false && Paper == false)
            {
                rbtFacturae.Checked = true;
            }
        }

        #endregion

        #region "Public Methods"


        /// <summary>
        /// Método que carga el archivo de idioma y establece los textos de los controles
        /// en el idioma que corresponde.
        /// </summary>
        public void LoadLanguage()
        {
            try
            {
                Language labels = Language.getInstance();
                this.Text = labels.getLabels(this.Name);
                gbxSelectedType.Text = labels.getLabels(gbxSelectedType.Name);
                lbSelectedType.Text = labels.getLabels(lbSelectedType.Name);
                rbtFacturae.Text = labels.getLabels(rbtFacturae.Name);
                rbtPaper.Text = labels.getLabels(rbtPaper.Name);
                btnCancel.Text = labels.getLabels(btnCancel.Name);
                btnOk.Text = labels.getLabels(btnOk.Name);
            }
            catch (Exception)
            {
                ControlMessages msg = ControlMessages.getInstance();
                MessageBox.Show(msg.getMessage(ControlMessages.Msg.ErrorLoadLanguage), msg.getMessage(ControlMessages.Msg.CaptionInvoiceType), MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        #endregion

        
    }
}
