// ----------------------------------------------------------------------------------------
// Author:                    Joaqun Lpez y Magally Vargas
// Company:                   Albalia Interactiva S.L
// Assembly version:          1.0.0.0
// Date:                      10/02/2009
// Time:                      12:06
// Solution Name:             FacturaeAdd-in
// Solution Filename:         FacturaeAdd-in.sln
// Solution FullFilename:     C:\Ribbon_Facturae\FacturaeAdd-in.sln
// Project Name:              FacturaeAdd-in
// Project Filename:          FacturaeAdd-in.csproj
// Project FullFilename:      C:\Ribbon_Facturae\FacturaeAdd-in\FacturaeAdd-in.csproj
// Project Item Name:         FrmSettings.cs
// Project Item Filename:     FrmSettings.cs
// Project Item FullFilename: C:\Ribbon_Facturae\FacturaeAdd-in\FrmSettings.cs
// Project Item Kind:         Formulario
// Purpose:                   Configurar la manera de emitir facturas segun las necesidades del emisor
//Copyright  2008 - 2009 Albalia Interactiva S.L. Reservados todos los derechos.
// ----------------------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Collections;
using System.IO;

namespace FacturaeAdd_in
{
    /// <summary>
    /// Formulario que permite al usuario configurar los distintos aspectos de las facturaes,
    /// apariencia, seguridad, datos corporativos....
    /// Dichos datos de configuracion son almaceandos en un archivo de configuracin XML.
    /// </summary>
    public partial class FrmSettings : Form
    {        
        #region "Properties"

        /// <summary>
        /// Propiedad de tipo ConfigData la cual contiene la informacin necesaria para rellenar los campos
        /// del formulario.
        /// Es la representacin del archivo XML de configuracin como un objeto .NET
        /// </summary>
        public ConfigData Configuration { get; set; }

        #endregion

        #region "Constructors"

        /// <summary>
        /// Constructor que inicializa todos los componentes grficos del formulario.
        /// </summary>
        public FrmSettings()
        {
            InitializeComponent();
        }

        #endregion

        #region "EventHandler"

        /// <summary>
        /// Mtodo que maneja el evento que se produce al hacer click con el ratn sobre el botn de aceptar.
        /// Comprueba que los datos sean correctos y guarda dichos datos en la propiedad de configuracin para
        /// mas tarde almacenarlos en el archivo de configuracin.
        /// Si el proceso ha ido bien se muestra un mensaje donde se dice que la nueva configuracin ha sido
        /// guardada correctamente, sino se mostrar el consiguiente mensaje de error.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnOk_Click(object sender, EventArgs e)
        {
            ControlMessages msg = ControlMessages.getInstance();
            if (EsValido())
            {
                if (rbtXadesXl.Checked)
                {
                    Configuration.SignedType = "XADES-XL";
                }
                else
                {
                    Configuration.SignedType = "XADES-EPES";
                }

                Configuration.UrlSTServer = txtUrlTimeStamp.Text;
                Configuration.UrlIssued = txtIssuedDirectory.Text;
                Configuration.UrlRecived = txtReceivedDirectory.Text;
                Configuration.UrlDrafts = txtDraftsDirectory.Text;

                setDefaultInvoicesPath();

                try
                {

                    XmlFile.Save(Configuration, Globals.ThisAddIn.EnvironmentVariable + Paths.ConfigPath, Configuration.GetType());
                    XmlFile.Save(Configuration, System.Environment.GetEnvironmentVariable("APPDATA") + @"\config.xml", Configuration.GetType());
                }
                catch (Exception)
                {
                    MessageBox.Show(this, String.Concat(msg.getMessage(ControlMessages.Msg.ErrorSaveConfig), "\n", msg.getMessage(ControlMessages.Msg.InfoRestart)), msg.getMessage(ControlMessages.Msg.CaptionConfig), MessageBoxButtons.OK, MessageBoxIcon.Error, MessageBoxDefaultButton.Button1);
                    return;
                }

                MessageBox.Show(this, msg.getMessage(ControlMessages.Msg.InfoSaveConfigOk), msg.getMessage(ControlMessages.Msg.CaptionConfig), MessageBoxButtons.OK, MessageBoxIcon.Information, MessageBoxDefaultButton.Button1);
                this.Close();
            }
        }

        /// <summary>
        /// Mtodo que maneja el evento que se produce al hacer click con el ratn sobre el botn de cancelar.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnCancel_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        /// <summary>
        /// Mtodo que maneja el evento que se produce al hacer click con el ratn sobre el botn de examinar
        /// el directorio de las facturas emitidas.
        /// Abre un cuadro de dilogo para buscar el directorio donde se quieren almacenar las facturaes emitidas.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnIssuedBrowser_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog f = new FolderBrowserDialog();
            if (!(String.IsNullOrEmpty(txtIssuedDirectory.Text)) && Directory.Exists(txtIssuedDirectory.Text))
            {
                f.SelectedPath = txtIssuedDirectory.Text;
            }
            if (f.ShowDialog() == DialogResult.OK)
            {
                txtIssuedDirectory.Text = f.SelectedPath;
                //Configuration.UrlIssued = f.SelectedPath;
            }
        }

        /// <summary>
        /// Mtodo que maneja el evento que se produce al hacer click con el ratn sobre el botn de examinar
        /// el directorio de las facturas recibidas.
        /// Abre un cuadro de dilogo para buscar el directorio donde se quieren almacenar las facturaes recibidas.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnRecivedBrowser_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog f = new FolderBrowserDialog();
            if (!(String.IsNullOrEmpty(txtReceivedDirectory.Text)) && Directory.Exists(txtReceivedDirectory.Text))
            {
                f.SelectedPath = txtReceivedDirectory.Text;
            }
            if (f.ShowDialog() == DialogResult.OK)
            {
                txtReceivedDirectory.Text = f.SelectedPath;
                //Configuration.UrlRecived = f.SelectedPath;
            }
        }

        /// <summary>
        /// Mtodo que maneja el evento que se produce al hacer click con el ratn sobre el botn de examinar
        /// el directorio de los borradores.
        /// Abre un cuadro de dilogo para buscar el directorio donde se quieren almacenar los borradores.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnDraftsBrowser_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog f = new FolderBrowserDialog();
            if (!(String.IsNullOrEmpty(txtDraftsDirectory.Text)) && Directory.Exists(txtDraftsDirectory.Text))
            {
                f.SelectedPath = txtDraftsDirectory.Text;
            }
            if (f.ShowDialog() == DialogResult.OK)
            {
                txtDraftsDirectory.Text = f.SelectedPath;
                //Configuration.UrlDrafts = f.SelectedPath;
            }
        }

        /// <summary>
        /// Mtodo que maneja el evento que se produce al hacer click con el ratn sobre el botn de empresa.
        /// Se muestra el formulario con los datos de la empresa.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnCompany_Click(object sender, EventArgs e)
        {
            FrmCompanyInformation frmCompany = new FrmCompanyInformation();
            frmCompany.Configuration = Configuration;
            frmCompany.Show();
        }

        private void lbCompanySettings_Click(object sender, EventArgs e)
        {
            
        }

        /// <summary>
        /// Mtodo que maneja el evento que se produce al cargar el formulario.
        /// Se inicializan los campos con los datos de la propiedad Configuracin, los 
        /// cuales estaban almacenados en el archivo de configuracin.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void FrmSettings_Load(object sender, EventArgs e)
        {
            LoadLanguage();
            LoadComboBox(Globals.ThisAddIn.EnvironmentVariable + Paths.UrlTimeStamp, txtUrlTimeStamp);
            if (Configuration.SignedType.Equals("XADES-XL"))
            {
                rbtXadesXl.Checked = true;
            }
            else
            {
                rbtXadesEpes.Checked = true;
            }

            string invoiceDirectory = Environment.GetFolderPath(Environment.SpecialFolder.Personal) + @"\Mis Facturas";

            setDefaultInvoicesPath();


            txtUrlTimeStamp.Text = Configuration.UrlSTServer;
            txtIssuedDirectory.Text = Configuration.UrlIssued;
            txtReceivedDirectory.Text = Configuration.UrlRecived;
            txtDraftsDirectory.Text = Configuration.UrlDrafts;
            txtTemplateType.Text = Configuration.UrlTemplate;
        }

        

        /// <summary>
        /// Mtodo que maneja el evento que se produce al hacer click con el ratn sobre el botn de personalizar.
        /// Se muestra el formulario con los datos de la empresa.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnTemplateTypeBrowser_Click(object sender, EventArgs e)
        {
            OpenFileDialog f = new OpenFileDialog();
            f.Filter = "Plantillas (*.dotx)|*.dotx|Documentos de Word (*.docx)|*.docx";
            f.RestoreDirectory = true;
            if (!(String.IsNullOrEmpty(Configuration.UrlTemplate)) && File.Exists(Configuration.UrlTemplate))
            {
                f.InitialDirectory = txtTemplateType.Text;
            }
            if (f.ShowDialog() == DialogResult.OK)
            {
                txtTemplateType.Text = f.FileName;
                Configuration.UrlTemplate = txtTemplateType.Text;
            }
        }



        /// <summary>
        /// Asigna el directorio de facturas por defecto, en caso de que no se hayan configurado aun o los dejen vacios
        /// </summary>
        private void setDefaultInvoicesPath()
        {
            try
            {
                string invoiceDirectory = Environment.GetFolderPath(Environment.SpecialFolder.Personal) + @"\Mis Facturas";

                if (string.IsNullOrEmpty(Configuration.UrlIssued))
                    Configuration.UrlIssued = invoiceDirectory + @"\Emitidas";
                if (string.IsNullOrEmpty(Configuration.UrlRecived))
                    Configuration.UrlRecived = invoiceDirectory + @"\Recibidas";
                if (string.IsNullOrEmpty(Configuration.UrlDrafts))
                    Configuration.UrlDrafts = invoiceDirectory + @"\Borradores";
            }
            catch (Exception) { }
        }

        #endregion

        #region "Public Methods"

        /// <summary>
        /// Mtodo que comprueba si los datos introducidos por el usuario son correctos.
        /// En caso negativo se muestra un mensaje informativo indicando aquellos datos que no son vlidos.
        /// </summary>
        /// <returns>Devuelve true o false dependiendo de si los datos son correctos o no</returns>
        public bool EsValido() 
        {
            ControlMessages cmsg = ControlMessages.getInstance();
            string msg = String.Concat(cmsg.getMessage(ControlMessages.Msg.InfoCorrectFields), "\n\n");
            bool correcto = true;
            if (string.IsNullOrEmpty(txtUrlTimeStamp.Text))
            {
                msg = string.Concat(msg, cmsg.getMessage(ControlMessages.Msg.InfoUrlSTM), "\n");
                correcto = false;
            }
            if (string.IsNullOrEmpty(txtIssuedDirectory.Text))
            {
                msg = string.Concat(msg, cmsg.getMessage(ControlMessages.Msg.InfoUrlIssuedDirectory), "\n");
                correcto = false;
            }
            if (string.IsNullOrEmpty(txtReceivedDirectory.Text))
            {
                msg = string.Concat(msg, cmsg.getMessage(ControlMessages.Msg.InfoUrlRecivedDirectory), "\n");
                correcto = false;
            }
            if (string.IsNullOrEmpty(txtDraftsDirectory.Text))
            {
                msg = string.Concat(msg, cmsg.getMessage(ControlMessages.Msg.InfoUrlDraftsDirectory), "\n");
                correcto = false;
            }
            if (correcto)
            {
                return true;
            }
            MessageBox.Show(this, msg, cmsg.getMessage(ControlMessages.Msg.CaptionConfig), MessageBoxButtons.OK, MessageBoxIcon.Information, MessageBoxDefaultButton.Button1);
            return false;
        }

        /// <summary>
        /// Mtodo que carga el archivo de idioma y establece los textos de los controles
        /// en el idioma que corresponde.
        /// </summary>
        public void LoadLanguage()
        {
            try
            {
                Language labels = Language.getInstance();
                this.Text = labels.getLabels(this.Name);
                gbxAppearance.Text = labels.getLabels(gbxAppearance.Name);
                gbxManagement.Text = labels.getLabels(gbxManagement.Name);
                gbxSecurity.Text = labels.getLabels(gbxSecurity.Name);
                gbxSignedType.Text = labels.getLabels(gbxSignedType.Name);
                gbxUrlTimeStamp.Text = labels.getLabels(gbxUrlTimeStamp.Name);

                btnCancel.Text = labels.getLabels(btnCancel.Name);
                btnCompany.Text = labels.getLabels(btnCompany.Name);
                
                btnDraftsBrowser.Text = labels.getLabels(btnDraftsBrowser.Name);
                btnIssuedBrowser.Text = labels.getLabels(btnIssuedBrowser.Name);
                btnOk.Text = labels.getLabels(btnOk.Name);
                btnRecivedBrowser.Text = labels.getLabels(btnRecivedBrowser.Name);
                btnTemplateTypeBrowser.Text = labels.getLabels(btnTemplateTypeBrowser.Name);

                lbCompanySettings.Text = labels.getLabels(lbCompanySettings.Name);
                lbDraftsDirectory.Text = labels.getLabels(lbDraftsDirectory.Name);
                lbIssuedDirectory.Text = labels.getLabels(lbIssuedDirectory.Name);
                lbRecivedDirectory.Text = labels.getLabels(lbRecivedDirectory.Name);
                lbTemplateType.Text = labels.getLabels(lbTemplateType.Name);
                
            }
            catch (Exception)
            {
                ControlMessages msg = ControlMessages.getInstance();
                MessageBox.Show(msg.getMessage(ControlMessages.Msg.ErrorLoadLanguage), msg.getMessage(ControlMessages.Msg.CaptionConfig), MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }


        /// <summary>
        /// Mtodo que se encarga de cargar un combobox a partir de un string con los datos.
        /// </summary>
        /// <param name="path">String que contiene el paht donde se encuentra el archivo con los valores que se van a cargar en el combo</param>
        /// <param name="cbx">Combobox que se desea cargar</param>
        public void LoadComboBox(string path, ComboBox cbx)
        {
            try
            {
                string names = ReadFile(path);
                string[] data = names.Split(',');
                for (int i = 0; i < data.Length; i++)
                {
                    cbx.Items.Add(data[i]);
                }
            }
            catch (Exception)
            {
                ControlMessages msg = ControlMessages.getInstance();
                MessageBox.Show(msg.getMessage(ControlMessages.Msg.ErrorLoadUrlTimeStamp), msg.getMessage(ControlMessages.Msg.CaptionCompanyInformation), MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// Mtodo que devuelve una cdena con los datos almacenados en el archivo especificado en 
        /// el path que recibe por parmetro.
        /// </summary>
        /// <param name="path">Ruta del archivo que vamos a leer</param>
        /// <returns>Un string con los datos en forma de cadena</returns>
        public string ReadFile(string path)
        {
            using (StreamReader sr = new StreamReader(path, Encoding.Default))
            {
                return sr.ReadToEnd();
            }
        }

        #endregion

       

        
    }
}
