﻿// ----------------------------------------------------------------------------------------
// Author:                    Joaquín López y Magally Vargas
// Company:                   Albalia Interactiva S.L
// Assembly version:          1.0.0.0
// Date:                      10/02/2009
// Time:                      12:14
// Solution Name:             FacturaeAdd-in
// Solution Filename:         FacturaeAdd-in.sln
// Solution FullFilename:     C:\Ribbon_Facturae\FacturaeAdd-in.sln
// Project Name:              FacturaeAdd-in
// Project Filename:          FacturaeAdd-in.csproj
// Project FullFilename:      C:\Ribbon_Facturae\FacturaeAdd-in\FacturaeAdd-in.csproj
// Project Item Name:         Ribbon1.cs
// Project Item Filename:     Ribbon1.cs
// Project Item FullFilename: C:\Ribbon_Facturae\FacturaeAdd-in\Ribbon1.cs
// Project Item Kind:         Componente
// Purpose:                   Ribbon con las opciones de Facturae
//Copyright © 2008 - 2009 Albalia Interactiva S.L. Reservados todos los derechos.
// ----------------------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Office.Tools.Ribbon;
using System.Windows.Forms;
using System.IO;
using System.Xml;
using FacturaeAdd_in.XmlMngFiles;
using System.Globalization;
using FacturaeAdd_in.FacturaeFiles;
using Microsoft.Office.Core;
using Microsoft.Office.Interop.Word;

namespace FacturaeAdd_in
{

    /// <summary>
    /// Ribbon que permite trabajar con facturae
    /// </summary>
    public partial class Ribbon1 : OfficeRibbon
    {

        #region "Fields"

        /// <summary>
        /// Variable de tipo bool que indica si existe o no un control
        /// </summary>
        private bool InvoiceControlExists;

        /// <summary>
        /// Variable que referencia a la aplicacion
        /// </summary>
        Microsoft.Office.Interop.Word.Application application;

        /// <summary>
        /// Variable que referencia al parser que se encarga de manejar los archivos de gestion de facturas
        /// </summary>
        XmlFileMngParser parser;

        /// <summary>
        /// Variable de tipo ConfiData, que una vez instanciada no se elimina hasta que se cierra la aplicación
        /// que contiene la configuración predeterminada por el usuario.De esta manera no se consulta constantemente
        /// el archivo xml que almacena esta configuración.
        /// </summary>
        ConfigData configuration;

        /// <summary>
        /// Variable de tipo string que indica el estado de una factura
        /// </summary>
        string status = "";

        /// <summary>
        /// Variables de tipo int que refencian a una numeración específica de las tablas de una planilla facturae
        /// Si alguna de estas referencias falla, significa qe no estamos trabajando con una plantilla válida.
        /// </summary>
        int t2 = 0; int t3 = 0; int t4 = 0; int t5 = 0; int t6 = 0; int t7 = 0;

        /// <summary>
        /// Variable de tipo char[] que contiene los caracteres que se deben elimiar cuando se cogen los valores
        /// de los controles de la plantilla.
        /// </summary>
        char[] x = { '\r', '\a', '\n' };

        #endregion

        #region "Constructors"

        /// <summary>
        /// Constructor que inicializa todos los componentes gráficos del Ribbon.
        /// </summary>
        public Ribbon1()
        {
            InitializeComponent();
        }

        #endregion

        #region "EventHandler"

        /// <summary>
        /// Método que maneja el evento que se produce al abrir el Ribbon.
        /// Se carga el idioma y se inicializa todo.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Ribbon1_Load(object sender, RibbonUIEventArgs e)
        {     
            LoadLanguage();
            application = Globals.ThisAddIn.Application;
            parser = new XmlFileMngParser();
            status=Globals.ThisAddIn.GetControlValue(FacturaeAdd_in.FacturaeFiles.ControlList.Tag.Status);
            if (!string.IsNullOrEmpty(status))
            {
                if (status.Equals("ISSUED"))
                    ckBoxIssued.Checked = true;
                else if (status.Equals("RECEIVED"))
                    ckBoxReceived.Checked = true;
                else
                    ckBoxDraft.Checked = true;
            }
            else
            {
                status = "DRAFT";
                ckBoxDraft.Checked = true;
            }
        }

        /// <summary>
        /// Método que maneja el evento que se produce al hacer click con el ratón sobre el botón de nueva factura.
        /// Se abre la plantilla predeterminada por el usuario en la configuracion con los datos del emisor cargados,
        /// también definidos en la configuracón.Si la plantilla no es válida, se abre una por defecto que si lo es.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnNew_Click(object sender, RibbonControlEventArgs e)
        {
                      
            LoadConfiguration();

            if (File.Exists(configuration.UrlTemplate))
            {
                object temp = configuration.UrlTemplate;
                Globals.ThisAddIn.LoadInvoiceTemplate(temp,true);
                Globals.ThisAddIn.LoadIssuerData(configuration);

            }
            else
            {
                ControlMessages msg = ControlMessages.getInstance();
                MessageBox.Show(msg.getMessage(ControlMessages.Msg.InfoNotExistDefaultTemplate), msg.getMessage(ControlMessages.Msg.CaptionConfig), MessageBoxButtons.OK, MessageBoxIcon.Information);
                try
                {
                    object temp = Globals.ThisAddIn.EnvironmentVariable + Paths.TemplatePath + Globals.ThisAddIn.TemplatesDirectories + "FacturaeBasica.dotx";
                    Globals.ThisAddIn.LoadInvoiceTemplate(temp,true);
                    Globals.ThisAddIn.LoadIssuerData(configuration);
                }
                catch (Exception)
                {

                }
            }
            status = "DRAFT";
            ckBoxDraft.Checked = true;
            ckBoxReceived.Checked = false;
            ckBoxIssued.Checked = false;
        }

        /// <summary>
        /// Método que maneja el evento que se produce al hacer click con el ratón sobre el botón de guardar.
        /// Muestra un cuadro de dialogo para guardar la facturae.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnSave_Click(object sender, RibbonControlEventArgs e)
        {
            try
            {
               
                       updateRegistry(saved(status),false,"");
               
            }
            catch (Exception)
            {
            }
            
         }

        

        /// <summary>
        /// Método que maneja el evento que se produce al hacer click con el ratón sobre el botón de emitir.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnIssue_Click(object sender, RibbonControlEventArgs e)
        {
            Emitir();
        }


        

        /// <summary>
        /// Método que maneja el evento que se produce al hacer click con el ratón sobre el botón de mis facturas.
        /// Muestra un TaskPane con los datos referentes a las facuras.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnMyInvoices_Click(object sender, RibbonControlEventArgs e)
         {

            if (!InvoiceControlExists)
            {

                Globals.ThisAddIn.OpenInvoiceTaskPane(this);
            }
            else
            {

                Globals.ThisAddIn.RemoveInvoiceTaskPane();
            }

            InvoiceControlExists = !InvoiceControlExists;
        }

        /// <summary>
        /// Método que maneja el evento que se produce al hacer click con el ratón sobre el botón de Configuración.
        /// Carga la cofiguración almacenada en el archivo XML y al mostrar el formulario de configuración le pasa
        /// la configuración.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnSettings_Click(object sender, RibbonControlEventArgs e)
        {

                LoadConfiguration();
                FrmSettings frmSettings = new FrmSettings();
                frmSettings.Configuration = configuration;
                frmSettings.Show();
            
        }

        /// <summary>
        /// Método que maneja el evento que se produce al hacer click con el ratón sobre el botón de emitir.
        /// Muestra un formulario donde el usuario selecciona el tipo de factura que desea realizar.
        /// Si es electrónica se llama a Emitir y si es en papel se llama a Cerrar.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnInvoiceType_Click(object sender, RibbonControlEventArgs e)
        {

            FrmInvoiceType f = new FrmInvoiceType();

            if (f.ShowDialog().Equals(DialogResult.OK)) 
            {


                if (f.Facturae)
                {
                    string id = Globals.ThisAddIn.GetControlValue(ControlList.Tag.customPartId);



                    CustomXMLPart facturaXMLPart = this.application.ActiveDocument.CustomXMLParts.SelectByID(id);

                    if (facturaXMLPart == null)
                    {

                        Emitir();
                    }
                    else
                    {
                        ControlMessages controlM = ControlMessages.getInstance();
                        MessageBox.Show(controlM.getMessage(ControlMessages.Msg.InvoiceAlReadySignated), controlM.getMessage(ControlMessages.Msg.CaptionFacturae), MessageBoxButtons.OK, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1);
                    }


                }
                else
                {
                    Cerrar();
                }
                
            }
            


        }

        /// <summary>
        /// Método que carga solo los datos de una factura en xml en la plantilla seleccionada por el usuario.
        /// Si no ha seleccionado ningina, los datos se cargarán en la plantilla que mejor se adapte.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btLoadXmlInvoice_Click(object sender, RibbonControlEventArgs e)
        {

            OpenFileDialog op = new OpenFileDialog();
            string filter = "Archivos Xml (*.xml)|*.xml";
            op.Filter = filter;

            if (op.ShowDialog().Equals(DialogResult.OK))
            {
                string file = op.FileName;

                Globals.ThisAddIn.LoadInvoice(file, false, false, true);

                Globals.ThisAddIn.setControlValue(ControlList.Tag.IH_InvoiceNumber, string.Empty);
                Globals.ThisAddIn.setControlValue(ControlList.Tag.CORR_InvoiceNumber, string.Empty);
                Globals.ThisAddIn.setControlValue(ControlList.Tag.CORR_InvoiceSeriesCode, string.Empty);


                Globals.ThisAddIn.enumerateTables();
                try
                {
                    Globals.ThisAddIn.Application.ActiveDocument.Tables[Globals.ThisAddIn.t4].Rows[2].Range.Text = string.Empty;
                }
                catch (Exception)
                {
                    
                }
                


                status = "DRAFT";
                ckBoxDraft.Checked = true;
                ckBoxReceived.Checked = false;
                ckBoxIssued.Checked = false;
            }
        }

        /// <summary>
        /// Método que abre una galeria de imágenes donde el usurio puede elegir la plantilla con la que desea trabajar.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void wordTemplatesGallery_Click(object sender, RibbonControlEventArgs e)
        {
            LoadConfiguration();
            object temp = Globals.ThisAddIn.EnvironmentVariable + Paths.TemplatePath + Globals.ThisAddIn.TemplatesDirectories + wordTemplatesGallery.SelectedItem.Tag.ToString() + ".dotx";
            
            Globals.ThisAddIn.LoadInvoiceTemplate(temp,true);
            if (configuration != null)
            {
                Globals.ThisAddIn.LoadIssuerData(configuration);
            }

            status = "DRAFT";
            ckBoxDraft.Checked = true;
            ckBoxReceived.Checked = false;
            ckBoxIssued.Checked = false;
            Globals.ThisAddIn.setControlValue(FacturaeAdd_in.FacturaeFiles.ControlList.Tag.Status, status);
        }

        
        
        

        /// <summary>
        /// Método que maneja el evento que se produce al hacer click sobre el botón de agregar a facturas recibidas.
        /// Se realiza una validacion de la factura, si todo está bien se añade a recibidas y se actualizan los archivos 
        /// de gestión.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnOpenRecivedFacturae_Click(object sender, RibbonControlEventArgs e)
        {
            ContentControl control = null;
            try
            {
                control = Globals.ThisAddIn.getControlByTag(ControlList.Tag.IH_InvoiceNumber);
            }
            catch (Exception) { }

            if (control != null)
            {

                try
                {
                    if (!Globals.ThisAddIn.validateReceivedInvoice(true))
                    {
                        return;
                    }
                    ckBoxIssued.Checked = false;
                    ckBoxDraft.Checked = false;
                    Globals.ThisAddIn.getControlByTag(ControlList.Tag.Status).LockContents = false;
                    status = "RECEIVED";
                    Globals.ThisAddIn.setControlValue(FacturaeAdd_in.FacturaeFiles.ControlList.Tag.Status, status);

                    Globals.ThisAddIn.getControlByTag(ControlList.Tag.Status).LockContents = true;
                    //Cargo la configuración porque puede darse el caso de que no este cargado todavia
                    LoadConfiguration();

                    string invType = "(P)";
                    string id = Globals.ThisAddIn.GetControlValue(ControlList.Tag.customPartId);
                    if (!string.IsNullOrEmpty(id))
                    {
                        CustomXMLPart customPart = Globals.ThisAddIn.Application.ActiveDocument.CustomXMLParts.SelectByID(id);
                        XmlDocument doc = new XmlDocument();
                        doc.PreserveWhitespace = true;
                        doc.LoadXml(customPart.DocumentElement.XML);
                        
                        XmlNode signatureNode = doc.DocumentElement.GetElementsByTagName("Signature").Item(0);
                        if (signatureNode == null)
                            signatureNode = doc.DocumentElement.GetElementsByTagName("ds:Signature").Item(0);
                        if (signatureNode != null)
                        {
                            invType = "(E)";
                        }
                        else
                        {
                            invType = "(P)";
                        }


                    }
                    updateRegistry(configuration.UrlRecived +@"\" + this.application.ActiveDocument.Name,true,invType);
                    ckBoxIssued.Checked = false;
                    ckBoxReceived.Checked = true;
                    ckBoxDraft.Checked = false;
                    Globals.ThisAddIn.updateTaskPane();
                    ControlMessages msg = ControlMessages.getInstance();
                    MessageBox.Show(msg.getMessage(ControlMessages.Msg.InfoAddRecived), msg.getMessage(ControlMessages.Msg.CaptionFacturae), MessageBoxButtons.OK, MessageBoxIcon.Information, MessageBoxDefaultButton.Button1);
                }
                catch (Exception exc)
                {
                    ControlMessages msg = ControlMessages.getInstance();
                    if (exc.Message.Equals("InfoInvoiceExist"))
                    {
                        MessageBox.Show(msg.getMessage(exc.Message), msg.getMessage(ControlMessages.Msg.CaptionRibbon), MessageBoxButtons.OK, MessageBoxIcon.Information);
                        return;
                    }

                    string error = string.Concat(msg.getMessage(ControlMessages.Msg.ErrorAddRecived), "\n", exc.Message);
                    MessageBox.Show(error, msg.getMessage(ControlMessages.Msg.CaptionFacturae), MessageBoxButtons.OK, MessageBoxIcon.Error, MessageBoxDefaultButton.Button1);
                }
            }
            else
            {
                ControlMessages controlM = ControlMessages.getInstance();
                MessageBox.Show(controlM.getMessage(ControlMessages.Msg.ErrorWhiteDocument), controlM.getMessage(ControlMessages.Msg.CaptionFacturae), MessageBoxButtons.OK, MessageBoxIcon.Information, MessageBoxDefaultButton.Button1);

            }

        }

        /// <summary>
        /// Método que maneja el evento que se produce al hacer click sobre el botón de cerrar factura
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnClose_Click(object sender, RibbonControlEventArgs e)
        {
            Cerrar();
        }


        

        /// <summary>
        /// Método que maneja el evento que se produce al hacer click sobre el checkboox de Emitida.
        /// Se llama al método que actualiza el estado.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ckBoxIssued_Click(object sender, RibbonControlEventArgs e)
        {
            updateStatusCheck();

        }

        /// <summary>
        /// Método que maneja el evento que se produce al hacer click sobre el checkboox de Recibida.
        /// Se llama al método que actualiza el estado.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ckBoxReceived_Click(object sender, RibbonControlEventArgs e)
        {
            
            updateStatusCheck();
        }

        /// <summary>
        /// Método que maneja el evento que se produce al hacer click sobre el checkboox de Borrador.
        /// Se llama al método que actualiza el estado.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ckBoxDraft_Click(object sender, RibbonControlEventArgs e)
        {
            updateStatusCheck();
        }

        /// <summary>
        /// Método que maneja el evento que se produce al hacer click sobre el botón de extraer factura.
        /// Abre un cuadro de dialogo donde el usuario elige donde quiere guardar la factura xml
        /// y se extrae la factura.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void button1_Click(object sender, RibbonControlEventArgs e)
        {
            SaveFileDialog sd = new SaveFileDialog();
            sd.Filter = "Archivos Xml (*.xml)|*.xml";
            sd.ShowDialog();
            string path = sd.FileName;
            try
            {
                Globals.ThisAddIn.extraerfacturae(path, true);
            }
            catch (Exception)
            {
                try
                {
                    Globals.ThisAddIn.exportPaperInvoice(path,true);
                }
                catch (Exception)
                {
                    ControlMessages controlM = ControlMessages.getInstance();
                    MessageBox.Show(controlM.getMessage(ControlMessages.Msg.ErrorInvoiceNotGenerated), controlM.getMessage(ControlMessages.Msg.CaptionFacturae), MessageBoxButtons.OK, MessageBoxIcon.Error, MessageBoxDefaultButton.Button1);

                }
            }
        }

        

        /// <summary>
        /// Método que maneja el evento que se produce al hacer click sobre el boton de validar factura.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btInvoiceValidation_Click(object sender, RibbonControlEventArgs e)
        {
            try
            {
                ContentControl control = null;
                try
                {
                    control = Globals.ThisAddIn.getControlByTag(ControlList.Tag.IH_InvoiceNumber);
                }
                catch (Exception) { }

                if (control != null)
                {
                    Globals.ThisAddIn.validateReceivedInvoice(false);
                }
                else
                {
                    ControlMessages controlM = ControlMessages.getInstance();
                    MessageBox.Show(controlM.getMessage(ControlMessages.Msg.ErrorWhiteDocument), controlM.getMessage(ControlMessages.Msg.CaptionFacturae), MessageBoxButtons.OK, MessageBoxIcon.Information, MessageBoxDefaultButton.Button1);

                }
            }
            catch (Exception)
            {
                ControlMessages controlM = ControlMessages.getInstance();
                MessageBox.Show(controlM.getMessage(ControlMessages.Msg.ErrorWhiteDocument), controlM.getMessage(ControlMessages.Msg.CaptionFacturae), MessageBoxButtons.OK, MessageBoxIcon.Information, MessageBoxDefaultButton.Button1);
            }
        }


        /// <summary>
        /// Método que maneja el evento que se produce cuando se hace click con el ratón sobre el
        /// boton de importar factura.
        /// Se abre un cuadro de dialogo donde el usuario seleccioana la factura xml que desea cargar,
        /// se realiza una validacion de la factura, si es válida se carga en la plantilla que mejor
        /// se ajuste, si no es válida se le informa al usuario y se le da la posibilidad de ver los
        /// resultados de la validación
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnImportReceivedInvoice_Click(object sender, RibbonControlEventArgs e)
        {

            OpenFileDialog op = new OpenFileDialog();
            string filter = "Archivos Xml (*.xml)|*.xml";
            op.Filter = filter;
            op.ShowDialog();
            string file = op.FileName;
            ControlMessages msg = ControlMessages.getInstance();
            bool schema = true;
            bool account = true;
            bool signed = true;

            if (!string.IsNullOrEmpty(file))
            {

                try
                {

                    try
                    {
                        Globals.ThisAddIn.LoadInvoice(file, false, true, true);
                    }
                    catch (Exception)
                    {
                        schema = false;
                        throw new Exception("1");
                    }

                    try
                    {
                        Globals.ThisAddIn.AccountingValidation(file, true);
                    }
                    catch (Exception)
                    {
                        account = false;
                    }

                    try
                    {
                        Globals.ThisAddIn.SignatureValidations(file);
                    }
                    catch (Exception)
                    {
                        signed = false;
                    }


                    try
                    {
                        string tempDir = Path.GetTempPath();

                        string data = Globals.ThisAddIn.GetXmlFromResource(file);
                        bool papel = true;
                        if (data.IndexOf("Signature") > 0)
                        {
                            papel = false;
                            Globals.ThisAddIn.AddCustomXmlPart(data, this.application.ActiveDocument);
                        }

                        string name = "";

                        try
                        {
                            name = string.Concat(Globals.ThisAddIn.GetControlValue(ControlList.Tag.SP_TaxIdentificationNumber), Globals.ThisAddIn.GetControlValue(ControlList.Tag.IH_InvoiceNumber), Globals.ThisAddIn.GetControlValue(ControlList.Tag.IH_InvoiceSeriesCode));
                        }
                        catch (Exception) { }

                        object filename;

                        if (string.IsNullOrEmpty(name))
                            filename = Path.GetTempFileName();
                        else
                            filename = tempDir + name;

                        Globals.ThisAddIn.setControlValue(ControlList.Tag.Status, "RECEIVED");
                        ckBoxReceived.Checked = true;
                        ckBoxIssued.Checked = false;
                        ckBoxDraft.Checked = false;
                        status = "RECEIVED";


                        object missing = System.Reflection.Missing.Value;

                        application.ActiveDocument.SaveAs(ref filename, ref missing, ref missing, ref missing, ref missing, ref missing,
                        ref missing, ref missing, ref missing, ref missing, ref missing, ref missing, ref missing, ref missing, ref missing, ref missing);
                        if (!schema || !account || (!signed && !papel))
                        {
                            string error = msg.getMessage(ControlMessages.Msg.ErrorSchema) + " " + msg.getMessage(ControlMessages.Msg.ErrorFails) + "\n";
                            if (!schema)
                                error += msg.getMessage(ControlMessages.Msg.MsgInfoSchema) + "\n";
                            if (!account)
                                error += msg.getMessage(ControlMessages.Msg.MsgAccounValidation) + "\n";
                            if (!signed)
                                error += msg.getMessage(ControlMessages.Msg.MsgSing) + "\n";
                            error += msg.getMessage(ControlMessages.Msg.MsgInfoViewDetails);
                            if (MessageBox.Show(error, msg.getMessage(ControlMessages.Msg.CaptionFacturae), MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                            {
                                try
                                {
                                    Globals.ThisAddIn.validateReceivedInvoice(false);
                                }
                                catch (Exception)
                                {
                                }

                            }
                        }
                        else
                        {
                            if (MessageBox.Show(msg.getMessage(ControlMessages.Msg.InfoImportOK), msg.getMessage(ControlMessages.Msg.CaptionFacturae), MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                            {
                                try
                                {
                                    Globals.ThisAddIn.validateReceivedInvoice(false);
                                }
                                catch (Exception)
                                {
                                }
                            }
                        }
                    }
                    catch (Exception) { }


                }
                catch (Exception exc)
                {
                    if (exc.Message.Equals("1"))
                    {
                        MessageBox.Show(msg.getMessage(ControlMessages.Msg.ErrorSchema), msg.getMessage(ControlMessages.Msg.CaptionFacturae), MessageBoxButtons.OK, MessageBoxIcon.Error, MessageBoxDefaultButton.Button1);
                        return;
                    }

                    MessageBox.Show(msg.getMessage(ControlMessages.Msg.ErrorCauses) + exc.Message, msg.getMessage(ControlMessages.Msg.CaptionFacturae), MessageBoxButtons.OK, MessageBoxIcon.Error, MessageBoxDefaultButton.Button1);
                }

            }

        }

        #endregion

        #region "Public Methods"

        

        /// <summary>
        /// Método que carga el archivo de idioma y establece los textos de los controles
        /// en el idioma que corresponde.
        /// </summary>
        public void LoadLanguage()
        {
            try
            {
                Language labels = Language.getInstance();
                btnNew.Label = labels.getLabels(btnNew.Name);
                btnOpenReceivedFacturae.Label = labels.getLabels(btnOpenReceivedFacturae.Name);
                btnSave.Label = labels.getLabels(btnSave.Name);
                btnInvoiceType.Label = labels.getLabels(btnInvoiceType.Name);              
                btnMyInvoices.Label = labels.getLabels(btnMyInvoices.Name);
                btLoadXmlInvoice.Label = labels.getLabels(btLoadXmlInvoice.Name);
                btExportToXml.Label = labels.getLabels(btExportToXml.Name);
                btnSettings.Label = labels.getLabels(btnSettings.Name);
                ckBoxIssued.Label = labels.getLabels(ckBoxIssued.Name);
                ckBoxReceived.Label = labels.getLabels(ckBoxReceived.Name);
                ckBoxDraft.Label = labels.getLabels(ckBoxDraft.Name);
                gbxFacturae.Label = labels.getLabels(gbxFacturae.Name);
                gbxTemplates.Label = labels.getLabels(gbxTemplates.Name);
                gpStatus.Label = labels.getLabels(gpStatus.Name);
                wordTemplatesGallery.Label = labels.getLabels(wordTemplatesGallery.Name);
                btInvoiceValidation.Label = labels.getLabels(btInvoiceValidation.Name);
                gbxXML.Label = labels.getLabels(gbxXML.Name);
                gbxReceivedInvoices.Label = labels.getLabels(gbxReceivedInvoices.Name);
                btnImportReceivedInvoice.Label = labels.getLabels(btnImportReceivedInvoice.Name);
                btnNew.ScreenTip = labels.getLabels(btnNew.Name);
                btnSave.ScreenTip = labels.getLabels(btnSave.Name);
                btnInvoiceType.ScreenTip = labels.getLabels(btnInvoiceType.Name);
                btnSettings.ScreenTip = labels.getLabels(btnSettings.Name);
                btnMyInvoices.ScreenTip = labels.getLabels(btnMyInvoices.Name);
                btnImportReceivedInvoice.ScreenTip = labels.getLabels(btnImportReceivedInvoice.Name);
                btInvoiceValidation.ScreenTip = labels.getLabels(btInvoiceValidation.Name);
                btnOpenReceivedFacturae.ScreenTip = labels.getLabels(btnOpenReceivedFacturae.Name);
                btLoadXmlInvoice.ScreenTip = labels.getLabels(btLoadXmlInvoice.Name);
                btExportToXml.ScreenTip = labels.getLabels(btExportToXml.Name);
                wordTemplatesGallery.ScreenTip = labels.getLabels(wordTemplatesGallery.Name);
                ckBoxDraft.ScreenTip = labels.getLabels("gbxStatusScreenTip");
                ckBoxIssued.ScreenTip = labels.getLabels("gbxStatusScreenTip");
                ckBoxReceived.ScreenTip = labels.getLabels("gbxStatusScreenTip");
                btnNew.SuperTip = labels.getLabels("btnNewSuperTip");
                btnSave.SuperTip = labels.getLabels("btnSaveSuperTip");
                btnInvoiceType.SuperTip = labels.getLabels("btnInvoiceTypeSuperTip");
                btnSettings.SuperTip = labels.getLabels("btnSettingsSuperTip");
                btnMyInvoices.SuperTip = labels.getLabels("btnMyInvoicesSuperTip");
                btnImportReceivedInvoice.SuperTip = labels.getLabels("btnImportReceivedInvoiceSuperTip");
                btInvoiceValidation.SuperTip = labels.getLabels("btInvoiceValidationSuperTip");
                btnOpenReceivedFacturae.SuperTip = labels.getLabels("btnOpenReceivedFacturaeSuperTip");
                btLoadXmlInvoice.SuperTip = labels.getLabels("btLoadXmlInvoiceSuperTip");
                btExportToXml.SuperTip = labels.getLabels("btExportToXmlSuperTip");
                wordTemplatesGallery.SuperTip = labels.getLabels("wordTemplatesGallerySuperTip");
                ckBoxDraft.SuperTip = labels.getLabels("gbxStatusSuperTip");
                ckBoxIssued.SuperTip = labels.getLabels("gbxStatusSuperTip");
                ckBoxReceived.SuperTip = labels.getLabels("gbxStatusSuperTip");

            
            }
            catch (Exception)
            {
                ControlMessages msg = ControlMessages.getInstance();
                MessageBox.Show(msg.getMessage(ControlMessages.Msg.ErrorLoadLanguage), msg.getMessage(ControlMessages.Msg.CaptionRibbon), MessageBoxButtons.OK, MessageBoxIcon.Error);
            }


        }

        /// <summary>
        /// Método que carga la configuración predeterminada por el usuario de un archivo xml
        /// </summary>
        public void LoadConfiguration() 
        {
            //Si no se ha cargado la configuración
            if (configuration == null) 
            {
                //Creo una nueva
                configuration = new ConfigData();
                //Si no se puede cargar la configuración
                if (!(XmlFile.LoadObjectFrom(ref configuration,Globals.ThisAddIn.EnvironmentVariable + Paths.ConfigPath)))
                {
                    //Pido al usuario si kiere configurarla o si no se creara uno en blanco
                    ControlMessages msg = ControlMessages.getInstance();
                    //Si si kiere le muestro el formulario de configuracion
                    if (MessageBox.Show(String.Concat(msg.getMessage(ControlMessages.Msg.ErrorLoadConfig), "\n", msg.getMessage(ControlMessages.Msg.InfoConfigModifyOrEmpty), "\n", msg.getMessage(ControlMessages.Msg.InfoConfigQuestion)), msg.getMessage(ControlMessages.Msg.CaptionConfig), MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                    {
                        //XmlFile.Save(config, pathConfig, config.GetType());
                        FrmSettings frmSettings = new FrmSettings();
                        frmSettings.Configuration = configuration;
                        frmSettings.Show();
                    }
                    //Sino creo uno en blanco
                    else 
                    {
                        XmlFile.Save(configuration, Globals.ThisAddIn.EnvironmentVariable + Paths.ConfigPath, configuration.GetType());
                    }
                }
            }
        }

        /// <summary>
        /// Método que actualiza los archivos de gestion de las facturas
        /// </summary>
        /// <param name="SavePath">Parámetro de tipo string que contiene el path físico donde se ha guardado la factura</param>
        /// <param name="agregarRecibidas">Parámetro de tipo bool que indica si es una factura que se quiere añadir a recibidas</param>
        /// <param name="InvoiceType">Parámetro de tipo string que indica el tipo de factura</param>
        public void updateRegistry(string SavePath, bool agregarRecibidas, string InvoiceType)
        {
            if (string.IsNullOrEmpty(SavePath))
            {
                return;
            }
            if (string.IsNullOrEmpty(status))
            {
                ControlMessages msg = ControlMessages.getInstance();
                throw new Exception(msg.getMessage(ControlMessages.Msg.InfoSaveWhiteDoc));
            }
            else
            {
                string path = SavePath;
                if (path.IndexOf(".docx") < 0)
                {
                    path = path + ".docx";
                }
                if (!path.Equals(string.Empty))
                {
                    try
                    {
                        string serie = Globals.ThisAddIn.GetControlValue(FacturaeAdd_in.FacturaeFiles.ControlList.Tag.IH_InvoiceSeriesCode);
                        string fecha = Globals.ThisAddIn.GetControlValue(FacturaeAdd_in.FacturaeFiles.ControlList.Tag.InvIssue_IssueDate);
                        string n_factura = Globals.ThisAddIn.GetControlValue(FacturaeAdd_in.FacturaeFiles.ControlList.Tag.IH_InvoiceNumber);
                        string cif = Globals.ThisAddIn.GetControlValue(FacturaeAdd_in.FacturaeFiles.ControlList.Tag.BP_TaxIdentificationNumber);
                        string customer = "";
                        string customerFirstName = "";
                        string customerSecondName = "";
                        string issuerName = "";
                        string issuerFirstName = "";
                        string issuerSecondName = "";

                        enumerateTables();

                        if (string.IsNullOrEmpty(serie) || string.IsNullOrEmpty(cif) || string.IsNullOrEmpty(n_factura) || Globals.ThisAddIn.GetControlValue(FacturaeAdd_in.FacturaeFiles.ControlList.Tag.BP_PersonTypeCode) == null)
                        {
                            if (!agregarRecibidas)
                            {
                                ControlMessages msg = ControlMessages.getInstance();
                                throw new Exception(msg.getMessage(ControlMessages.Msg.InfoCheckDraft));
                            }

                        }

                        if (controlValue(ControlList.Tag.BP_PersonTypeCode,getControlValueByTag(ControlList.Tag.BP_PersonTypeCode)).ToLower().Equals("f"))
                        {
                            customer = Globals.ThisAddIn.GetControlValue(FacturaeAdd_in.FacturaeFiles.ControlList.Tag.BP_Name);
                            customerFirstName = Globals.ThisAddIn.GetControlValue(FacturaeAdd_in.FacturaeFiles.ControlList.Tag.BP_FirstSurname);
                            customerSecondName = Globals.ThisAddIn.GetControlValue(FacturaeAdd_in.FacturaeFiles.ControlList.Tag.BP_SecondSurname);
                        }
                        else
                            customer = Globals.ThisAddIn.GetControlValue(FacturaeAdd_in.FacturaeFiles.ControlList.Tag.BP_CorporateName);

                        if (controlValue(ControlList.Tag.SP_PersonTypeCode, getControlValueByTag(ControlList.Tag.SP_PersonTypeCode)).ToLower().Equals("f"))
                        {
                            issuerName = Globals.ThisAddIn.GetControlValue(FacturaeAdd_in.FacturaeFiles.ControlList.Tag.SP_Name);
                            issuerFirstName = Globals.ThisAddIn.GetControlValue(FacturaeAdd_in.FacturaeFiles.ControlList.Tag.SP_FirstSurname);
                            issuerSecondName = Globals.ThisAddIn.GetControlValue(FacturaeAdd_in.FacturaeFiles.ControlList.Tag.SP_SecondSurname);
                        }
                        else
                            issuerName = Globals.ThisAddIn.GetControlValue(FacturaeAdd_in.FacturaeFiles.ControlList.Tag.SP_CorporateName);



                        string importeT = Globals.ThisAddIn.Application.ActiveDocument.Tables[t5].Rows.Last.Cells[3].Range.Text.Trim(x).Trim();

                        CultureInfo cultDate = new CultureInfo("es-ES");
                        DateTime issueDate = DateTime.Parse(fecha, cultDate);
                        


                        parser.addNewXmlInvoiceRecord(issueDate.Year.ToString(), issueDate.Month.ToString(), status, new XmlInvoice(serie, n_factura, cif, customer, customerFirstName, customerSecondName, issuerName, issuerFirstName, issuerSecondName, fecha, importeT, status, "", path, InvoiceType));

                        object filename = path; object missing = System.Reflection.Missing.Value;



                        application.ActiveDocument.SaveAs(ref filename, ref missing, ref missing, ref missing, ref missing, ref missing,
                        ref missing, ref missing, ref missing, ref missing, ref missing, ref missing, ref missing, ref missing, ref missing, ref missing);
                        Globals.ThisAddIn.updateTaskPane();
                    }
                    catch (Exception e)
                    {
                        ControlMessages msg = ControlMessages.getInstance();
                        if (e.Message.Equals("InfoInvoiceExist"))
                        {
                            throw e;
                        }
                        MessageBox.Show(e.Message, msg.getMessage(ControlMessages.Msg.CaptionRibbon), MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }
            }
        }

        /// <summary>
        /// Método que comprueba si el documento activo es una plantilla válida de facturae.
        /// Para ello se basa en la numeración fija de las tablas.
        /// </summary>
        public void enumerateTables()
        {
            try
            {
                Microsoft.Office.Interop.Word.Document document = Globals.ThisAddIn.Application.ActiveDocument;

                int count = document.Tables.Count;
                if (count <= 0)
                {
                    ControlMessages msg = ControlMessages.getInstance();
                    throw new Exception(msg.getMessage(ControlMessages.Msg.ErrorNotValidTemplate));
                }
                else
                {

                    for (int i = 2; i <= document.Tables.Count; i++)
                    {
                        try
                        {
                            string table = document.Tables[i].Rows[1].Cells[1].Range.Text.Trim(x).Trim();
                            switch (table)
                            {

                                case "2":
                                    t2 = i; break;
                                case "3":
                                    t3 = i; break;
                                case "6":
                                    t4 = i; break;
                                case "4":
                                    t5 = i; break;
                                case "1":
                                    t6 = i; break;
                                case "7":
                                    t7 = i; break;

                            }
                        }
                        catch (Exception) { }
                    }
                }
            }
            catch (Exception) { }

        }

        /// <summary>
        /// Método que actualiza los check de estado
        /// </summary>
        public void updateStatusCheck()
        {
            bool i = false;
            bool r = false;
            bool d = false;
            if (status.Equals("ISSUED"))
            {
                i = true;
            }
            if (status.Equals("RECEIVED"))
            {
                r = true;
            }
            if (status.Equals("DRAFT"))
            {
                d = true;
            }
            ckBoxIssued.Checked = i;
            ckBoxReceived.Checked = r;
            ckBoxDraft.Checked = d;
        }

        /// <summary>
        /// Método que cierra una factua en papel
        /// Igual que el metodo de emitir pero sin firma.
        /// </summary>
        public void Cerrar()
        {
            LoadConfiguration();
            SaveFileDialog op = new SaveFileDialog();
            op.Filter = "Arhivos de Word 2007 (*.docx)|*.docx";
            op.InitialDirectory = setInitiaDirectory("ISSUED");
            string name = string.Concat(Globals.ThisAddIn.GetControlValue(FacturaeAdd_in.FacturaeFiles.ControlList.Tag.SP_TaxIdentificationNumber), Globals.ThisAddIn.GetControlValue(FacturaeAdd_in.FacturaeFiles.ControlList.Tag.IH_InvoiceNumber), Globals.ThisAddIn.GetControlValue(FacturaeAdd_in.FacturaeFiles.ControlList.Tag.IH_InvoiceSeriesCode));
            op.FileName = name;
            if (op.ShowDialog() == DialogResult.Cancel)
            {
                return;
            }
            string path = Globals.ThisAddIn.EnvironmentVariable + Paths.CustomXmlPartPath;

            if (!string.IsNullOrEmpty(op.FileName))
            {
                try
                {

                    Globals.ThisAddIn.cerrarFactura(path, configuration);


                    ckBoxIssued.Checked = true;
                    ckBoxReceived.Checked = false;
                    ckBoxDraft.Checked = false;
                    status = "ISSUED";
                    Globals.ThisAddIn.setControlValue(FacturaeAdd_in.FacturaeFiles.ControlList.Tag.Status, status);


                    string serie = Globals.ThisAddIn.GetControlValue(FacturaeAdd_in.FacturaeFiles.ControlList.Tag.IH_InvoiceSeriesCode);
                    string n_factura = Globals.ThisAddIn.GetControlValue(FacturaeAdd_in.FacturaeFiles.ControlList.Tag.IH_InvoiceNumber);
                    string cif = Globals.ThisAddIn.GetControlValue(FacturaeAdd_in.FacturaeFiles.ControlList.Tag.BP_TaxIdentificationNumber);

                    //Añado la factura al registro de emitidos.

                    updateRegistry(op.FileName, false, "(P)");
                    object readOnly = true; object missing = System.Reflection.Missing.Value;
                    object filename = op.FileName;

                    this.application.ActiveDocument.SaveAs(ref filename, ref missing, ref missing, ref missing, ref missing, ref missing, ref readOnly, ref missing, ref missing, ref missing, ref missing, ref missing, ref missing, ref missing, ref missing, ref missing);
                    ControlMessages msg = ControlMessages.getInstance();
                    MessageBox.Show(msg.getMessage(ControlMessages.Msg.InfoGeneratedInvoice), msg.getMessage(ControlMessages.Msg.CaptionFacturae), MessageBoxButtons.OK, MessageBoxIcon.Information);
                   
                    string fecha = Globals.ThisAddIn.GetControlValue(FacturaeAdd_in.FacturaeFiles.ControlList.Tag.InvIssue_IssueDate);
                    CultureInfo cultDate = new CultureInfo("es-ES");
                    DateTime issueDate = DateTime.Parse(fecha, cultDate);
                    //Elimino el borrador
                    parser.removeInvoive(serie, n_factura, cif, Globals.ThisAddIn.EnvironmentVariable + Paths.DraftPath + issueDate.Year.ToString() + issueDate.Month.ToString() + ".xml");
                    Globals.ThisAddIn.updateTaskPane();
                }
                catch (Exception ex)
                {
                    ControlMessages msg = ControlMessages.getInstance();
                    if (ex.Message.Equals("InfoInvoiceExist"))
                    {
                        MessageBox.Show(msg.getMessage(ex.Message), msg.getMessage(ControlMessages.Msg.CaptionRibbon), MessageBoxButtons.OK, MessageBoxIcon.Information);
                        return;
                    }

                    MessageBox.Show(ex.Message, msg.getMessage(ControlMessages.Msg.CaptionRibbon), MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        /// <summary>
        /// Metodo que muestra al usuario el cuadro de dialogo para que seleccione donde desea 
        /// guardar la factura
        /// </summary>
        /// <param name="status"></param>
        /// <returns></returns>
        public string saved(string status)
        {

            if (string.IsNullOrEmpty(this.application.ActiveDocument.Path))
            {
                SaveFileDialog f = new SaveFileDialog();
                f.Filter = "Todos los archivos (*.*)|*.*|Documentos de Word (*.docx)|*.docx|Archivos XML (*.xml)|*.xml";
                f.RestoreDirectory = true;
                f.InitialDirectory = setInitiaDirectory(status);
                string name = string.Concat(Globals.ThisAddIn.GetControlValue(FacturaeAdd_in.FacturaeFiles.ControlList.Tag.SP_TaxIdentificationNumber), "_", Globals.ThisAddIn.GetControlValue(FacturaeAdd_in.FacturaeFiles.ControlList.Tag.IH_InvoiceNumber), "_", Globals.ThisAddIn.GetControlValue(FacturaeAdd_in.FacturaeFiles.ControlList.Tag.IH_InvoiceSeriesCode), "_", "(Draft)");
                f.FileName = name;
                if (f.ShowDialog().Equals(DialogResult.Cancel))
                {
                    throw new Exception();
                }
                return f.FileName;
            }
            else
            {
                application.ActiveDocument.Save();
                updateRegistry(this.application.ActiveDocument.Path + @"\" + this.application.ActiveDocument.Name, false, "");
                return string.Empty;
            }


        }

        /// <summary>
        /// Método que devuelve el path donde se encuentran las facturas que deterina el estado.
        /// </summary>
        /// <param name="status"></param>
        /// <returns></returns>
        public string setInitiaDirectory(string status)
        {

            LoadConfiguration();
            if (status.ToUpper().Equals("ISSUED".ToUpper()))
                return configuration.UrlIssued;
            else
                if (status.ToUpper().Equals("RECEIVED".ToUpper()))
                    return configuration.UrlRecived;
                else
                    return configuration.UrlDrafts;



        }

        /// <summary>
        /// Método que emite una factura.
        /// Abre el cuadro de diálogo para que el usuario indique donde quiere guardar la factura,
        /// por defecto será en la carpeta de facturas emitidas definida en la configuración, despues
        /// se llama a emiir factura donde se genera la factura, se hace la validacion contable, se firma
        /// y se valdia la firma.Si todo va bien, se actualizan los archivos de gestión, agregando la factura
        /// al archivo de emitidas y eliminandolo de borradore si exisiera.
        /// </summary>
        private void Emitir()
        {
            Globals.ThisAddIn.LoadCountryComboBox();
            LoadConfiguration();
            SaveFileDialog op = new SaveFileDialog();
            op.Filter = "Arhivos de Word 2007 (*.docx)|*.docx";
            op.InitialDirectory = setInitiaDirectory("ISSUED");
            string name = string.Concat(Globals.ThisAddIn.GetControlValue(FacturaeAdd_in.FacturaeFiles.ControlList.Tag.SP_TaxIdentificationNumber), Globals.ThisAddIn.GetControlValue(FacturaeAdd_in.FacturaeFiles.ControlList.Tag.IH_InvoiceNumber), Globals.ThisAddIn.GetControlValue(FacturaeAdd_in.FacturaeFiles.ControlList.Tag.IH_InvoiceSeriesCode));
            op.FileName = name;

            if (op.ShowDialog() == DialogResult.Cancel)
            {
                return;
            }
            string path = Globals.ThisAddIn.EnvironmentVariable + Paths.CustomXmlPartPath;
            string pathSignate = Globals.ThisAddIn.EnvironmentVariable + Paths.SignateCustomXmlPartPath;

            if (!string.IsNullOrEmpty(op.FileName))
            {
                try
                {


                    Globals.ThisAddIn.emitirFactura(path, pathSignate, configuration);


                    ckBoxIssued.Checked = true;
                    ckBoxReceived.Checked = false;
                    ckBoxDraft.Checked = false;
                    status = "ISSUED";

                    Globals.ThisAddIn.setControlValue(FacturaeAdd_in.FacturaeFiles.ControlList.Tag.Status, status);

                    string serie = Globals.ThisAddIn.GetControlValue(FacturaeAdd_in.FacturaeFiles.ControlList.Tag.IH_InvoiceSeriesCode);
                    string n_factura = Globals.ThisAddIn.GetControlValue(FacturaeAdd_in.FacturaeFiles.ControlList.Tag.IH_InvoiceNumber);
                    string cif = Globals.ThisAddIn.GetControlValue(FacturaeAdd_in.FacturaeFiles.ControlList.Tag.BP_TaxIdentificationNumber);


                    //Añado la factura al registro de emitidos.
                    updateRegistry(op.FileName, false, "(E)");

                    object readOnly = true; object missing = System.Reflection.Missing.Value;
                    object filename = op.FileName;


                    string data = Globals.ThisAddIn.GetXmlFromResource(pathSignate);
                    Globals.ThisAddIn.AddCustomXmlPart(data, Globals.ThisAddIn.Application.ActiveDocument);


                    this.application.ActiveDocument.Save();

                    string fecha = Globals.ThisAddIn.GetControlValue(FacturaeAdd_in.FacturaeFiles.ControlList.Tag.InvIssue_IssueDate);
                    CultureInfo cultDate = new CultureInfo("es-ES");
                    DateTime issueDate = DateTime.Parse(fecha, cultDate);

                    //Elimino el borrador
                    parser.removeInvoive(serie, n_factura, cif, Globals.ThisAddIn.EnvironmentVariable + Paths.DraftPath + issueDate.Year.ToString() + issueDate.Month.ToString() + ".xml");


                    Globals.ThisAddIn.updateTaskPane();

                    ControlMessages msg = ControlMessages.getInstance();
                    MessageBox.Show(msg.getMessage(ControlMessages.Msg.InfoGeneratedInvoice), msg.getMessage(ControlMessages.Msg.CaptionFacturae), MessageBoxButtons.OK, MessageBoxIcon.Information);

                }
                catch (Exception ex)
                {
                    ControlMessages msg = ControlMessages.getInstance();

                    if (ex.Message.Equals("ErrorCertificateNull") || ex.Message.Equals("NotSigned"))
                        return;
                    if (ex.Message.Equals("ErrorCheckSignature"))
                    {
                        MessageBox.Show(msg.getMessage(ControlMessages.Msg.InfoInvoiceValidation), msg.getMessage(ControlMessages.Msg.CaptionRibbon), MessageBoxButtons.OK, MessageBoxIcon.Error);
                        return;
                    }

                    if (ex.Message.Equals("ErrorCorrectiveReason"))
                    {
                        MessageBox.Show(msg.getMessage(ControlMessages.Msg.ErrorCorrectiveReason), msg.getMessage(ControlMessages.Msg.CaptionRibbon), MessageBoxButtons.OK, MessageBoxIcon.Error);
                        return;
                    }

                    if (ex.Message.Equals("ErrorCorrectiveMethod"))
                    {
                        MessageBox.Show(msg.getMessage(ControlMessages.Msg.ErrorCorrectiveMethod), msg.getMessage(ControlMessages.Msg.CaptionRibbon), MessageBoxButtons.OK, MessageBoxIcon.Error);
                        return;
                    }

                    if (ex.Message.Equals("InfoInvoiceExist"))
                    {
                        MessageBox.Show(msg.getMessage(ex.Message), msg.getMessage(ControlMessages.Msg.CaptionRibbon), MessageBoxButtons.OK, MessageBoxIcon.Information);
                        return;
                    }
                    MessageBox.Show(ex.Message, msg.getMessage(ControlMessages.Msg.CaptionRibbon), MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        /// <summary>
        /// Método que devuelve una referencia al checkbox de Emitida
        /// </summary>
        /// <returns></returns>
        public RibbonCheckBox getCkBoxIssued()
        {
            return ckBoxIssued;
        }

        /// <summary>
        /// Método que devuelve una referencia al checkbox de Recibida
        /// </summary>
        /// <returns></returns>
        public RibbonCheckBox getCkBoxReceived()
        {
            return ckBoxReceived;
        }

        /// <summary>
        /// Método que devuelve una referencia al checkbox de Borrador
        /// </summary>
        /// <returns></returns>
        public RibbonCheckBox getCkBoxDraft()
        {
            return ckBoxDraft;
        }

        /// <summary>
        /// Método que cambia el valor de la variable de estado por el que recibe por parámetro.
        /// </summary>
        /// <param name="status">Parámetro de tipo string con el nuevo estado</param>
        public void setStatus(string status) 
        {
            this.status = status;
        }

        /// <summary>
        ///Busca el ContentControl con el nombre especificado y devuelve el texto digitado
        /// </summary>
        /// <param name="tag">enumerado de tipo ControlList.tag que representa el nombre del contentControl</param>
        /// <returns>el Valor del contentControl con nombre = tag</returns>
        public string getControlValueByTag(ControlList.Tag tag)
        {
            string value = null;
            if (string.IsNullOrEmpty(tag.ToString("g")))
                return value;
            foreach (ContentControl item in Globals.ThisAddIn.Application.ActiveDocument.ContentControls)
            {
                if (!string.IsNullOrEmpty(item.Tag))
                {
                    if (item.Tag.Equals(tag.ToString("g")))
                    {
                        if (string.IsNullOrEmpty(item.Range.Text))
                            return value;
                        value = item.Range.Text.Trim();
                        break;
                    }
                }
            }

            return value;


        }
        private string controlValue(ControlList.Tag tag, string selectedItem)
        {
            string value = "";

            if (string.IsNullOrEmpty(tag.ToString("g")))
                return value;

            /*buscamos el combobox indicado*/
            foreach (ContentControl item in Globals.ThisAddIn.Application.ActiveDocument.ContentControls)
            {
                if (!string.IsNullOrEmpty(item.Tag))
                {
                    if (item.Tag.Equals(tag.ToString("g")))
                    {
                        /*si lo encontramos buscamos el valor correspondiente al texto seleccionado*/
                        foreach (ContentControlListEntry entry in item.DropdownListEntries)
                        {
                            if (!string.IsNullOrEmpty(item.Tag))
                            {
                                if (entry.Text.ToLower().Equals(selectedItem.ToLower()))
                                {
                                    value = entry.Value; break;
                                }
                            }

                        } break;
                    }
                }
            }
            return value;
        }



        #endregion

    }
}
